#    Copyright 2019 Michael Johnson
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Spares pool table

Revision ID: 6ffc710674ef
Revises: 7432f1d4ea83
Create Date: 2019-03-11 10:45:43.296236

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '6ffc710674ef'
down_revision = '7432f1d4ea83'


def upgrade():
    op.create_table(
        u'spares_pool',
        sa.Column(u'updated_at', sa.DateTime(), primary_key=True,
                  server_default=sa.func.current_timestamp()))



#    Copyright (c) 2018 OpenStack Foundation
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os

import jinja2
from octavia_lib.common import constants as lib_consts

from octavia.common.config import cfg
from octavia.common import constants
from octavia.common import utils as octavia_utils


CONF = cfg.CONF

PROTOCOL_MAP = {
    constants.PROTOCOL_UDP: 'udp',
    lib_consts.PROTOCOL_SCTP: 'sctp'
}

BALANCE_MAP = {
    constants.LB_ALGORITHM_ROUND_ROBIN: 'wrr',
    constants.LB_ALGORITHM_LEAST_CONNECTIONS: 'lc',
    constants.LB_ALGORITHM_SOURCE_IP: 'sh'
}

BASE_PATH = CONF.haproxy_amphora.base_path

CHECK_SCRIPT_NAME = 'udp_check.sh'

KEEPALIVED_LVS_TEMPLATE = os.path.abspath(
    os.path.join(os.path.dirname(__file__),
                 'templates/keepalivedlvs.cfg.j2'))

JINJA_ENV = None


class LvsJinjaTemplater(object):

    def __init__(self, base_amp_path=None, keepalivedlvs_template=None):
        """Keepalived LVS configuration generation

        :param base_amp_path: Base path for amphora data
        :param keepalivedlvs_template: Absolute path to Jinja template
        """

        self.base_amp_path = base_amp_path or BASE_PATH
        self.keepalivedlvs_template = (keepalivedlvs_template or
                                       KEEPALIVED_LVS_TEMPLATE)

    def build_config(self, listener, **kwargs):
        """Convert a logical configuration to the Keepalived LVS version

        :param listener: The listener configuration
        :return: Rendered configuration
        """
        return self.render_loadbalancer_obj(listener)

    def _get_template(self):
        """Returns the specified Jinja configuration template."""
        global JINJA_ENV
        if not JINJA_ENV:
            template_loader = jinja2.FileSystemLoader(
                searchpath=os.path.dirname(self.keepalivedlvs_template))
            JINJA_ENV = jinja2.Environment(
                autoescape=True,
                loader=template_loader,
                trim_blocks=True,
                lstrip_blocks=True,
                extensions=['jinja2.ext.do'])
        return JINJA_ENV.get_template(os.path.basename(
            self.keepalivedlvs_template))

    def render_loadbalancer_obj(self, listener, **kwargs):
        """Renders a templated configuration from a load balancer object

        :param host_amphora: The Amphora this configuration is hosted on
        :param listener: The listener configuration
        :return: Rendered configuration
        """
        loadbalancer = self._transform_loadbalancer(
            listener.load_balancer,
            listener)
        return self._get_template().render(
            {'loadbalancer': loadbalancer},
            constants=constants,
            lib_consts=lib_consts)

    def _transform_loadbalancer(self, loadbalancer, listener):
        """Transforms a load balancer into an object that will

           be processed by the templating system
        """
        t_listener = self._transform_listener(listener)
        ret_value = {
            'id': loadbalancer.id,
            'vip_address': loadbalancer.vip.ip_address,
            'listener': t_listener,
            'enabled': loadbalancer.enabled
        }
        return ret_value

    def _transform_listener(self, listener):
        """Transforms a listener into an object that will

            be processed by the templating system
        """
        ret_value = {
            'id': listener.id,
            'protocol_port': listener.protocol_port,
            'protocol_mode': PROTOCOL_MAP[listener.protocol],
            'enabled': listener.enabled
        }
        if listener.connection_limit and listener.connection_limit > -1:
            ret_value['connection_limit'] = listener.connection_limit
        if (listener.default_pool and
            listener.default_pool.provisioning_status !=
                constants.PENDING_DELETE):
            ret_value['default_pool'] = self._transform_pool(
                listener.default_pool)
        return ret_value

    def _transform_pool(self, pool):
        """Transforms a pool into an object that will

            be processed by the templating system
        """
        ret_value = {
            'id': pool.id,
            'protocol': PROTOCOL_MAP[pool.protocol],
            'lb_algorithm': (BALANCE_MAP.get(pool.lb_algorithm) or
                             BALANCE_MAP[constants.LB_ALGORITHM_ROUND_ROBIN]),
            'members': [],
            'health_monitor': '',
            'session_persistence': '',
            'enabled': pool.enabled
        }
        members_gen = (mem for mem in pool.members if
                       mem.provisioning_status != constants.PENDING_DELETE)
        members = [self._transform_member(x) for x in members_gen]
        ret_value['members'] = members
        if (pool.health_monitor and
            pool.health_monitor.provisioning_status !=
                constants.PENDING_DELETE):
            ret_value['health_monitor'] = self._transform_health_monitor(
                pool.health_monitor)
        if pool.session_persistence:
            func = self._transform_session_persistence
            ret_value['session_persistence'] = func(
                pool.session_persistence)
        return ret_value

    @staticmethod
    def _transform_session_persistence(persistence):
        """Transforms session persistence into an object that will

            be processed by the templating system
        """
        return {
            'type': persistence.type,
            'persistence_timeout': persistence.persistence_timeout,
            'persistence_granularity': persistence.persistence_granularity
        }

    @staticmethod
    def _transform_member(member):
        """Transforms a member into an object that will

            be processed by the templating system
        """
        return {
            'id': member.id,
            'address': member.ip_address,
            'protocol_port': member.protocol_port,
            'weight': member.weight,
            'enabled': member.enabled,
            'monitor_address': member.monitor_address,
            'monitor_port': member.monitor_port
        }

    def _get_default_lvs_check_script_path(self, monitor_type):
        if monitor_type == constants.HEALTH_MONITOR_UDP_CONNECT:
            return (CONF.haproxy_amphora.base_path +
                    '/lvs/check/' + CHECK_SCRIPT_NAME)
        if monitor_type == lib_consts.HEALTH_MONITOR_SCTP:
            return "amphora-health-checker sctp"
        return None

    def _transform_health_monitor(self, monitor):
        """Transforms a health monitor into an object that will

            be processed by the templating system
        """
        return_val = {
            'id': monitor.id,
            'type': monitor.type,
            'delay': monitor.delay,
            'timeout': monitor.timeout,
            'enabled': monitor.enabled,
            'fall_threshold': monitor.fall_threshold,
            'check_script_path': (
                self._get_default_lvs_check_script_path(monitor.type))
        }
        if monitor.type == constants.HEALTH_MONITOR_HTTP:
            return_val.update({
                'rise_threshold': monitor.rise_threshold,
                'url_path': monitor.url_path,
                'http_method': (monitor.http_method
                                if monitor.http_method ==
                                constants.HEALTH_MONITOR_HTTP_METHOD_GET else
                                None),
                'expected_codes': (sorted(list(
                                   octavia_utils.expand_expected_codes(
                                       monitor.expected_codes)))
                                   if monitor.expected_codes else [])})
        return return_val



#    Copyright 2017 GoDaddy
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""modernize l7rule

Revision ID: d85ca7258d21
Revises: 034b2dc2f3e0
Create Date: 2017-04-04 06:26:55.287198

"""

from alembic import op
import sqlalchemy as sa

from octavia.common import constants

# revision identifiers, used by Alembic.
revision = 'd85ca7258d21'
down_revision = '034b2dc2f3e0'


def upgrade():
    # Add timing data
    op.add_column(
        u'l7rule',
        sa.Column(u'created_at', sa.DateTime(), nullable=True)
    )
    op.add_column(
        u'l7rule',
        sa.Column(u'updated_at', sa.DateTime(), nullable=True)
    )

    # Add project_id
    op.add_column(
        u'l7rule',
        sa.Column(u'project_id', sa.String(36), nullable=True)
    )

    # Add enabled
    op.add_column(
        u'l7rule',
        sa.Column(u'enabled', sa.Boolean(),
                  server_default=sa.sql.expression.true(),
                  nullable=False)
    )

    # Add new operating_status column, setting existing rows to ONLINE
    op.add_column(
        u'l7rule',
        sa.Column(u'operating_status', sa.String(16),
                  nullable=False, server_default=constants.ONLINE)
    )
    # Remove the default, as we don't actually want one
    op.alter_column(u'l7rule', u'operating_status',
                    existing_type=sa.String(16), server_default=None)
    # Add the foreign key for operating_status_name
    op.create_foreign_key(
        u'fk_l7rule_operating_status_name', u'l7rule',
        u'operating_status', [u'operating_status'], [u'name']
    )



#    Copyright 2015 Hewlett-Packard Development Company, L.P.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import signal
from unittest import mock

from octavia.cmd import health_manager
from octavia.tests.unit import base


class TestHealthManagerCMD(base.TestCase):

    def setUp(self):
        super().setUp()

    @mock.patch('multiprocessing.Event')
    @mock.patch('octavia.amphorae.drivers.health.'
                'heartbeat_udp.UDPStatusGetter')
    def test_hm_listener(self, mock_getter,
                         mock_event):
        mock_event.is_set.side_effect = [False, False, True]
        getter_mock = mock.MagicMock()
        check_mock = mock.MagicMock()
        getter_mock.check = check_mock
        getter_mock.check.side_effect = [None, Exception('break')]
        mock_getter.return_value = getter_mock
        health_manager.hm_listener(mock_event)
        mock_getter.assert_called_once()
        self.assertEqual(2, getter_mock.check.call_count)

    @mock.patch('multiprocessing.Event')
    @mock.patch('futurist.periodics.PeriodicWorker.start')
    @mock.patch('futurist.periodics.PeriodicWorker.__init__')
    @mock.patch('signal.signal')
    @mock.patch('octavia.controller.healthmanager.'
                'health_manager.HealthManager')
    def test_hm_health_check(self, mock_health, mock_signal, mock_worker,
                             mock_start, mock_event):
        mock_event.is_set.side_effect = [False, True]
        hm_mock = mock.MagicMock()
        mock_worker.return_value = None
        health_check_mock = mock.MagicMock()
        hm_mock.health_check = health_check_mock
        mock_health.return_value = hm_mock
        health_manager.hm_health_check(mock_event)
        mock_health.assert_called_once_with(mock_event)

    @mock.patch('multiprocessing.Process')
    @mock.patch('octavia.common.service.prepare_service')
    def test_main(self, mock_service, mock_process):
        mock_listener_proc = mock.MagicMock()
        mock_health_proc = mock.MagicMock()

        mock_process.side_effect = [mock_listener_proc, mock_health_proc]

        health_manager.main()

        mock_listener_proc.start.assert_called_once_with()
        mock_health_proc.start.assert_called_once_with()
        mock_listener_proc.join.assert_called_once_with()
        mock_health_proc.join.assert_called_once_with()

    @mock.patch('os.kill')
    @mock.patch('multiprocessing.Process')
    @mock.patch('octavia.common.service.prepare_service')
    def test_main_keyboard_interrupt(self, mock_service, mock_process,
                                     mock_kill):
        mock_listener_proc = mock.MagicMock()
        mock_health_proc = mock.MagicMock()
        mock_join = mock.MagicMock()
        mock_join.side_effect = [KeyboardInterrupt, None]
        mock_listener_proc.join = mock_join

        mock_process.side_effect = [mock_listener_proc, mock_health_proc]

        health_manager.main()

        mock_listener_proc.start.assert_called_once_with()
        mock_health_proc.start.assert_called_once_with()
        self.assertEqual(2, mock_listener_proc.join.call_count)
        mock_health_proc.join.assert_called_once_with()
        mock_kill.assert_called_once_with(mock_health_proc.pid,
                                          signal.SIGINT)

    @mock.patch('os.kill')
    @mock.patch('oslo_config.cfg.CONF.mutate_config_files')
    def test_handle_mutate_config(self, mock_mutate, mock_kill):
        health_manager._handle_mutate_config(1, 2)

        mock_mutate.assert_called_once()

        calls = [mock.call(1, signal.SIGHUP), mock.call(2, signal.SIGHUP)]
        mock_kill.assert_has_calls(calls)



# Copyright 2020 Red Hat, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc


class ImageBase(object, metaclass=abc.ABCMeta):

    @abc.abstractmethod
    def get_image_id_by_tag(self, image_tag, image_owner=None):
        """Get image ID by image tag and owner.

        :param image_tag: image tag
        :param image_owner: optional image owner
        :raises: ImageGetException if no images found with given tag
        :return: image id
        """



# Copyright 2020 Red Hat, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import errno
import os
import socket
from unittest import mock

import pyroute2

from octavia.amphorae.backends.utils import interface
from octavia.amphorae.backends.utils import interface_file
from octavia.common import constants as consts
from octavia.common import exceptions
from octavia.tests.common import utils as test_utils
import octavia.tests.unit.base as base


class TestInterface(base.TestCase):
    @mock.patch('os.listdir')
    @mock.patch('octavia.amphorae.backends.utils.interface_file.'
                'InterfaceFile.get_directory')
    def test_interface_file_list(self, mock_get_directory, mock_listdir):
        mock_get_directory.return_value = consts.AMP_NET_DIR_TEMPLATE

        ifaces = ('eth0', 'eth7', 'eth8')
        mock_listdir.return_value = [
            "{}.json".format(iface)
            for iface in ifaces
        ]
        mock_listdir.return_value.extend(["invalidfile"])

        controller = interface.InterfaceController()
        r = controller.interface_file_list()
        config_file_list = list(r)

        for iface in ifaces:
            f = os.path.join(consts.AMP_NET_DIR_TEMPLATE,
                             "{}.json".format(iface))
            self.assertIn(f, config_file_list)

        # unsupported file
        f = os.path.join(consts.AMP_NET_DIR_TEMPLATE,
                         "invalidfile")
        self.assertNotIn(f, config_file_list)

        # non existing file
        f = os.path.join(consts.AMP_NET_DIR_TEMPLATE,
                         "eth2.json")
        self.assertNotIn(f, config_file_list)

    @mock.patch('os.listdir')
    @mock.patch('octavia.amphorae.backends.utils.interface_file.'
                'InterfaceFile.get_directory')
    def test_list(self, mock_get_directory, mock_listdir):
        mock_get_directory.return_value = consts.AMP_NET_DIR_TEMPLATE
        mock_listdir.return_value = ["fakeiface.json"]

        content = ('{\n'
                   '"addresses": [\n'
                   '{"address": "10.0.0.2",\n'
                   '"prefixlen": 24}\n'
                   '],\n'
                   '"mtu": 1450,\n'
                   '"name": "eth1",\n'
                   '"routes": [\n'
                   '{"dst": "0.0.0.0/0",\n'
                   '"gateway": "10.0.0.1"},\n'
                   '{"dst": "10.11.0.0/16",\n'
                   '"gateway": "10.0.0.24"}\n'
                   '],\n'
                   '"rules": [\n'
                   '{"src": "10.0.0.2",\n'
                   '"src_len": 32,\n'
                   '"table": 100}\n'
                   '],\n'
                   '"scripts": {\n'
                   '"up": [\n'
                   '{"command": "up-script"}],\n'
                   '"down": [\n'
                   '{"command": "down-script"}]\n'
                   '}}\n')

        filename = os.path.join(consts.AMP_NET_DIR_TEMPLATE,
                                "fakeiface.json")

        self.useFixture(
            test_utils.OpenFixture(filename,
                                   contents=content))

        controller = interface.InterfaceController()
        ifaces = controller.list()

        self.assertIn("eth1", ifaces)
        iface = ifaces["eth1"]

        expected_dict = {
            consts.NAME: "eth1",
            consts.MTU: 1450,
            consts.ADDRESSES: [{
                consts.ADDRESS: "10.0.0.2",
                consts.PREFIXLEN: 24
            }],
            consts.ROUTES: [{
                consts.DST: "0.0.0.0/0",
                consts.GATEWAY: "10.0.0.1"
            }, {
                consts.DST: "10.11.0.0/16",
                consts.GATEWAY: "10.0.0.24"
            }],
            consts.RULES: [{
                consts.SRC: "10.0.0.2",
                consts.SRC_LEN: 32,
                consts.TABLE: 100
            }],
            consts.SCRIPTS: {
                consts.IFACE_UP: [{
                    consts.COMMAND: "up-script"
                }],
                consts.IFACE_DOWN: [{
                    consts.COMMAND: "down-script"
                }]
            }
        }

        self.assertEqual(expected_dict[consts.NAME], iface.name)
        self.assertEqual(expected_dict[consts.MTU], iface.mtu)
        test_utils.assert_address_lists_equal(
            self, expected_dict[consts.ADDRESSES], iface.addresses)
        test_utils.assert_rule_lists_equal(
            self, expected_dict[consts.RULES], iface.rules)
        test_utils.assert_script_lists_equal(
            self, expected_dict[consts.SCRIPTS], iface.scripts)

    def test__ipr_command(self):
        mock_ipr_addr = mock.MagicMock()

        controller = interface.InterfaceController()
        controller._ipr_command(mock_ipr_addr,
                                controller.ADD,
                                arg1=1, arg2=2)

        mock_ipr_addr.assert_called_once_with('add', arg1=1, arg2=2)

    def test__ipr_command_add_eexist(self):
        mock_ipr_addr = mock.MagicMock()
        mock_ipr_addr.side_effect = [
            pyroute2.NetlinkError(code=errno.EEXIST)
        ]

        controller = interface.InterfaceController()
        controller._ipr_command(mock_ipr_addr,
                                controller.ADD,
                                arg1=1, arg2=2)

        mock_ipr_addr.assert_called_once_with('add', arg1=1, arg2=2)

    def test__ipr_command_add_retry(self):
        mock_ipr_addr = mock.MagicMock()
        mock_ipr_addr.side_effect = [
            pyroute2.NetlinkError(code=errno.EINVAL),
            pyroute2.NetlinkError(code=errno.EINVAL),
            pyroute2.NetlinkError(code=errno.EINVAL),
            None
        ]

        controller = interface.InterfaceController()
        controller._ipr_command(mock_ipr_addr,
                                controller.ADD,
                                retry_on_invalid_argument=True,
                                retry_interval=0,
                                arg1=1, arg2=2)

        mock_ipr_addr.assert_has_calls([
            mock.call('add', arg1=1, arg2=2),
            mock.call('add', arg1=1, arg2=2),
            mock.call('add', arg1=1, arg2=2),
            mock.call('add', arg1=1, arg2=2)])

    def test__ipr_command_add_einval_failed(self):
        mock_ipr_addr = mock.MagicMock()
        mock_ipr_addr.__name__ = "addr"
        mock_ipr_addr.side_effect = [
            pyroute2.NetlinkError(code=errno.EINVAL)
        ] * 21

        controller = interface.InterfaceController()
        self.assertRaises(exceptions.AmphoraNetworkConfigException,
                          controller._ipr_command,
                          mock_ipr_addr,
                          controller.ADD,
                          retry_on_invalid_argument=True,
                          max_retries=20,
                          retry_interval=0,
                          arg1=1, arg2=2)
        mock_ipr_addr.assert_has_calls([
            mock.call('add', arg1=1, arg2=2)
        ] * 20)

    def test__ipr_command_add_failed(self):
        mock_ipr_addr = mock.MagicMock()
        mock_ipr_addr.__name__ = "addr"
        mock_ipr_addr.side_effect = [
            pyroute2.NetlinkError(code=errno.ENOENT)
        ]

        controller = interface.InterfaceController()
        self.assertRaises(exceptions.AmphoraNetworkConfigException,
                          controller._ipr_command,
                          mock_ipr_addr,
                          controller.ADD,
                          retry_on_invalid_argument=True,
                          max_retries=20,
                          retry_interval=0,
                          arg1=1, arg2=2)
        mock_ipr_addr.assert_called_once_with(
            'add', arg1=1, arg2=2)

    def test__ipr_command_delete_failed_no_raise(self):
        mock_ipr_addr = mock.MagicMock()
        mock_ipr_addr.__name__ = "addr"
        mock_ipr_addr.side_effect = [
            pyroute2.NetlinkError(code=errno.EINVAL)
        ]

        controller = interface.InterfaceController()
        controller._ipr_command(mock_ipr_addr,
                                controller.DELETE,
                                retry_on_invalid_argument=True,
                                max_retries=0,
                                raise_on_error=False,
                                arg1=1, arg2=2)
        mock_ipr_addr.assert_called_once_with(
            'delete', arg1=1, arg2=2)

    def test__ipr_command_add_failed_retry_no_raise(self):
        mock_ipr_addr = mock.MagicMock()
        mock_ipr_addr.__name__ = "addr"
        mock_ipr_addr.side_effect = [
            pyroute2.NetlinkError(code=errno.ENOENT)
        ]

        controller = interface.InterfaceController()
        controller._ipr_command(mock_ipr_addr,
                                controller.ADD,
                                max_retries=20,
                                retry_interval=0,
                                raise_on_error=False,
                                arg1=1, arg2=2)
        mock_ipr_addr.assert_called_once_with(
            'add', arg1=1, arg2=2)

    @mock.patch('subprocess.check_output')
    def test__dhclient_up(self, mock_check_output):
        iface = "iface2"

        controller = interface.InterfaceController()
        controller._dhclient_up(iface)

        mock_check_output.assert_called_once_with(
            ["/sbin/dhclient",
             "-lf",
             "/var/lib/dhclient/dhclient-{}.leases".format(
                 iface),
             "-pf",
             "/run/dhclient-{}.pid".format(iface),
             iface], stderr=-2)

    @mock.patch('subprocess.check_output')
    def test__dhclient_down(self, mock_check_output):
        iface = "iface2"

        controller = interface.InterfaceController()
        controller._dhclient_down(iface)

        mock_check_output.assert_called_once_with(
            ["/sbin/dhclient",
             "-r",
             "-lf",
             "/var/lib/dhclient/dhclient-{}.leases".format(
                 iface),
             "-pf",
             "/run/dhclient-{}.pid".format(iface),
             iface], stderr=-2)

    @mock.patch('subprocess.check_output')
    def test__ipv6auto_up(self, mock_check_output):
        iface = "iface2"

        controller = interface.InterfaceController()
        controller._ipv6auto_up(iface)

        mock_check_output.assert_has_calls([
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.iface2.accept_ra=2"], stderr=-2),
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.iface2.autoconf=1"], stderr=-2)])

    @mock.patch('subprocess.check_output')
    def test__ipv6auto_down(self, mock_check_output):
        iface = "iface2"

        controller = interface.InterfaceController()
        controller._ipv6auto_down(iface)

        mock_check_output.assert_has_calls([
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.iface2.accept_ra=0"], stderr=-2),
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.iface2.autoconf=0"], stderr=-2)])

    @mock.patch('pyroute2.IPRoute.rule')
    @mock.patch('pyroute2.IPRoute.route')
    @mock.patch('pyroute2.IPRoute.addr')
    @mock.patch('pyroute2.IPRoute.link')
    @mock.patch('pyroute2.IPRoute.link_lookup')
    @mock.patch('subprocess.check_output')
    def test_up(self, mock_check_output, mock_link_lookup, mock_link,
                mock_addr, mock_route, mock_rule):
        iface = interface_file.InterfaceFile(
            name="eth1",
            mtu=1450,
            addresses=[{
                consts.ADDRESS: '1.2.3.4',
                consts.PREFIXLEN: 24
            }, {
                consts.ADDRESS: '10.2.3.4',
                consts.PREFIXLEN: 16
            }, {
                consts.ADDRESS: '2001:db8::3',
                consts.PREFIXLEN: 64
            }],
            routes=[{
                consts.DST: '10.0.0.0/8',
                consts.GATEWAY: '1.0.0.1',
                consts.TABLE: 10,
                consts.ONLINK: True
            }, {
                consts.DST: '20.0.0.0/8',
                consts.GATEWAY: '1.0.0.2',
                consts.PREFSRC: '1.2.3.4',
                consts.SCOPE: 'link'
            }, {
                consts.DST: '2001:db8:2::1/128',
                consts.GATEWAY: '2001:db8::1'
            }],
            rules=[{
                consts.SRC: '1.1.1.1',
                consts.SRC_LEN: 32,
                consts.TABLE: 20,
            }, {
                consts.SRC: '2001:db8::1',
                consts.SRC_LEN: 128,
                consts.TABLE: 40,
            }],
            scripts={
                consts.IFACE_UP: [{
                    consts.COMMAND: "post-up eth1"
                }],
                consts.IFACE_DOWN: [{
                    consts.COMMAND: "post-down eth1"
                }],
            })

        idx = mock.MagicMock()
        mock_link_lookup.return_value = [idx]

        controller = interface.InterfaceController()
        controller.up(iface)

        mock_link.assert_called_once_with(
            controller.SET,
            index=idx,
            state=consts.IFACE_UP,
            mtu=1450)

        mock_addr.assert_has_calls([
            mock.call(controller.ADD,
                      index=idx,
                      address='1.2.3.4',
                      prefixlen=24,
                      family=socket.AF_INET),
            mock.call(controller.ADD,
                      index=idx,
                      address='10.2.3.4',
                      prefixlen=16,
                      family=socket.AF_INET),
            mock.call(controller.ADD,
                      index=idx,
                      address='2001:db8::3',
                      prefixlen=64,
                      family=socket.AF_INET6)
        ])

        mock_route.assert_has_calls([
            mock.call(controller.ADD,
                      oif=idx,
                      dst='10.0.0.0/8',
                      gateway='1.0.0.1',
                      table=10,
                      onlink=True,
                      family=socket.AF_INET),
            mock.call(controller.ADD,
                      oif=idx,
                      dst='20.0.0.0/8',
                      gateway='1.0.0.2',
                      prefsrc='1.2.3.4',
                      scope='link',
                      family=socket.AF_INET),
            mock.call(controller.ADD,
                      oif=idx,
                      dst='2001:db8:2::1/128',
                      gateway='2001:db8::1',
                      family=socket.AF_INET6)])

        mock_rule.assert_has_calls([
            mock.call(controller.ADD,
                      src="1.1.1.1",
                      src_len=32,
                      table=20,
                      family=socket.AF_INET),
            mock.call(controller.ADD,
                      src="2001:db8::1",
                      src_len=128,
                      table=40,
                      family=socket.AF_INET6)])

        mock_check_output.assert_has_calls([
            mock.call(["post-up", "eth1"])
        ])

    @mock.patch('pyroute2.IPRoute.rule')
    @mock.patch('pyroute2.IPRoute.route')
    @mock.patch('pyroute2.IPRoute.addr')
    @mock.patch('pyroute2.IPRoute.link')
    @mock.patch('pyroute2.IPRoute.link_lookup')
    @mock.patch('subprocess.check_output')
    def test_up_auto(self, mock_check_output, mock_link_lookup, mock_link,
                     mock_addr, mock_route, mock_rule):
        iface = interface_file.InterfaceFile(
            name="eth1",
            mtu=1450,
            addresses=[{
                consts.DHCP: True,
                consts.IPV6AUTO: True
            }],
            routes=[],
            rules=[],
            scripts={
                consts.IFACE_UP: [{
                    consts.COMMAND: "post-up eth1"
                }],
                consts.IFACE_DOWN: [{
                    consts.COMMAND: "post-down eth1"
                }],
            })

        idx = mock.MagicMock()
        mock_link_lookup.return_value = [idx]

        controller = interface.InterfaceController()
        controller.up(iface)

        mock_link.assert_called_once_with(
            controller.SET,
            index=idx,
            state=consts.IFACE_UP,
            mtu=1450)

        mock_addr.assert_not_called()
        mock_route.assert_not_called()
        mock_rule.assert_not_called()

        mock_check_output.assert_has_calls([
            mock.call(["/sbin/dhclient",
                       "-lf",
                       "/var/lib/dhclient/dhclient-{}.leases".format(
                           iface.name),
                       "-pf",
                       "/run/dhclient-{}.pid".format(iface.name),
                       iface.name], stderr=-2),
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.{}.accept_ra=2".format(iface.name)],
                      stderr=-2),
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.{}.autoconf=1".format(iface.name)],
                      stderr=-2),
            mock.call(["post-up", iface.name])
        ])

    @mock.patch('pyroute2.IPRoute.rule')
    @mock.patch('pyroute2.IPRoute.route')
    @mock.patch('pyroute2.IPRoute.addr')
    @mock.patch('pyroute2.IPRoute.link')
    @mock.patch('pyroute2.IPRoute.get_links')
    @mock.patch('pyroute2.IPRoute.link_lookup')
    @mock.patch('subprocess.check_output')
    def test_down(self, mock_check_output, mock_link_lookup, mock_get_links,
                  mock_link, mock_addr, mock_route, mock_rule):
        iface = interface_file.InterfaceFile(
            name="eth1",
            mtu=1450,
            addresses=[{
                consts.ADDRESS: '1.2.3.4',
                consts.PREFIXLEN: 24
            }, {
                consts.ADDRESS: '10.2.3.4',
                consts.PREFIXLEN: 16
            }, {
                consts.ADDRESS: '2001:db8::3',
                consts.PREFIXLEN: 64
            }],
            routes=[{
                consts.DST: '10.0.0.0/8',
                consts.GATEWAY: '1.0.0.1',
                consts.TABLE: 10,
                consts.ONLINK: True
            }, {
                consts.DST: '20.0.0.0/8',
                consts.GATEWAY: '1.0.0.2',
                consts.PREFSRC: '1.2.3.4',
                consts.SCOPE: 'link'
            }, {
                consts.DST: '2001:db8:2::1/128',
                consts.GATEWAY: '2001:db8::1'
            }],
            rules=[{
                consts.SRC: '1.1.1.1',
                consts.SRC_LEN: 32,
                consts.TABLE: 20,
            }, {
                consts.SRC: '2001:db8::1',
                consts.SRC_LEN: 128,
                consts.TABLE: 40,
            }],
            scripts={
                consts.IFACE_UP: [{
                    consts.COMMAND: "post-up eth1"
                }],
                consts.IFACE_DOWN: [{
                    consts.COMMAND: "post-down eth1"
                }],
            })

        idx = mock.MagicMock()
        mock_link_lookup.return_value = [idx]

        mock_get_links.return_value = [{
            consts.STATE: consts.IFACE_UP
        }]

        controller = interface.InterfaceController()
        controller.down(iface)

        mock_link.assert_called_once_with(
            controller.SET,
            index=idx,
            state=consts.IFACE_DOWN)

        mock_addr.assert_has_calls([
            mock.call(controller.DELETE,
                      index=idx,
                      address='1.2.3.4',
                      prefixlen=24,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      index=idx,
                      address='10.2.3.4',
                      prefixlen=16,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      index=idx,
                      address='2001:db8::3',
                      prefixlen=64,
                      family=socket.AF_INET6)
        ])

        mock_route.assert_has_calls([
            mock.call(controller.DELETE,
                      oif=idx,
                      dst='10.0.0.0/8',
                      gateway='1.0.0.1',
                      table=10,
                      onlink=True,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      oif=idx,
                      dst='20.0.0.0/8',
                      gateway='1.0.0.2',
                      prefsrc='1.2.3.4',
                      scope='link',
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      oif=idx,
                      dst='2001:db8:2::1/128',
                      gateway='2001:db8::1',
                      family=socket.AF_INET6)])

        mock_rule.assert_has_calls([
            mock.call(controller.DELETE,
                      src="1.1.1.1",
                      src_len=32,
                      table=20,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      src="2001:db8::1",
                      src_len=128,
                      table=40,
                      family=socket.AF_INET6)])

        mock_check_output.assert_has_calls([
            mock.call(["post-down", "eth1"])
        ])

    @mock.patch('pyroute2.IPRoute.rule')
    @mock.patch('pyroute2.IPRoute.route')
    @mock.patch('pyroute2.IPRoute.addr')
    @mock.patch('pyroute2.IPRoute.link')
    @mock.patch('pyroute2.IPRoute.get_links')
    @mock.patch('pyroute2.IPRoute.link_lookup')
    @mock.patch('subprocess.check_output')
    def test_down_with_errors(self, mock_check_output, mock_link_lookup,
                              mock_get_links, mock_link, mock_addr,
                              mock_route, mock_rule):
        iface = interface_file.InterfaceFile(
            name="eth1",
            mtu=1450,
            addresses=[{
                consts.ADDRESS: '1.2.3.4',
                consts.PREFIXLEN: 24
            }, {
                consts.ADDRESS: '10.2.3.4',
                consts.PREFIXLEN: 16
            }, {
                consts.ADDRESS: '2001:db8::3',
                consts.PREFIXLEN: 64
            }],
            routes=[{
                consts.DST: '10.0.0.0/8',
                consts.GATEWAY: '1.0.0.1',
                consts.TABLE: 10,
                consts.ONLINK: True
            }, {
                consts.DST: '20.0.0.0/8',
                consts.GATEWAY: '1.0.0.2',
                consts.PREFSRC: '1.2.3.4',
                consts.SCOPE: 'link'
            }, {
                consts.DST: '2001:db8:2::1/128',
                consts.GATEWAY: '2001:db8::1'
            }],
            rules=[{
                consts.SRC: '1.1.1.1',
                consts.SRC_LEN: 32,
                consts.TABLE: 20,
            }, {
                consts.SRC: '2001:db8::1',
                consts.SRC_LEN: 128,
                consts.TABLE: 40,
            }],
            scripts={
                consts.IFACE_UP: [{
                    consts.COMMAND: "post-up eth1"
                }],
                consts.IFACE_DOWN: [{
                    consts.COMMAND: "post-down eth1"
                }],
            })

        idx = mock.MagicMock()
        mock_link_lookup.return_value = [idx]

        mock_get_links.return_value = [{
            consts.STATE: consts.IFACE_UP
        }]
        mock_addr.side_effect = [
            pyroute2.NetlinkError(123),
            pyroute2.NetlinkError(123),
            pyroute2.NetlinkError(123)
        ]
        mock_route.side_effect = [
            pyroute2.NetlinkError(123),
            pyroute2.NetlinkError(123),
            pyroute2.NetlinkError(123)
        ]
        mock_rule.side_effect = [
            pyroute2.NetlinkError(123),
            pyroute2.NetlinkError(123),
        ]
        mock_check_output.side_effect = [
            Exception()
        ]

        controller = interface.InterfaceController()
        controller.down(iface)

        mock_link.assert_called_once_with(
            controller.SET,
            index=idx,
            state=consts.IFACE_DOWN)

        mock_addr.assert_has_calls([
            mock.call(controller.DELETE,
                      index=idx,
                      address='1.2.3.4',
                      prefixlen=24,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      index=idx,
                      address='10.2.3.4',
                      prefixlen=16,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      index=idx,
                      address='2001:db8::3',
                      prefixlen=64,
                      family=socket.AF_INET6)
        ])

        mock_route.assert_has_calls([
            mock.call(controller.DELETE,
                      oif=idx,
                      dst='10.0.0.0/8',
                      gateway='1.0.0.1',
                      table=10,
                      onlink=True,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      oif=idx,
                      dst='20.0.0.0/8',
                      gateway='1.0.0.2',
                      prefsrc='1.2.3.4',
                      scope='link',
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      oif=idx,
                      dst='2001:db8:2::1/128',
                      gateway='2001:db8::1',
                      family=socket.AF_INET6)])

        mock_rule.assert_has_calls([
            mock.call(controller.DELETE,
                      src="1.1.1.1",
                      src_len=32,
                      table=20,
                      family=socket.AF_INET),
            mock.call(controller.DELETE,
                      src="2001:db8::1",
                      src_len=128,
                      table=40,
                      family=socket.AF_INET6)])

        mock_check_output.assert_has_calls([
            mock.call(["post-down", "eth1"])
        ])

    @mock.patch('pyroute2.IPRoute.rule')
    @mock.patch('pyroute2.IPRoute.route')
    @mock.patch('pyroute2.IPRoute.addr')
    @mock.patch('pyroute2.IPRoute.link')
    @mock.patch('pyroute2.IPRoute.get_links')
    @mock.patch('pyroute2.IPRoute.link_lookup')
    @mock.patch('subprocess.check_output')
    def test_down_already_down(self, mock_check_output, mock_link_lookup,
                               mock_get_links, mock_link, mock_addr,
                               mock_route, mock_rule):
        iface = interface_file.InterfaceFile(
            name="eth1",
            mtu=1450,
            addresses=[{
                consts.ADDRESS: '1.2.3.4',
                consts.PREFIXLEN: 24
            }, {
                consts.ADDRESS: '10.2.3.4',
                consts.PREFIXLEN: 16
            }, {
                consts.ADDRESS: '2001:db8::3',
                consts.PREFIXLEN: 64
            }],
            routes=[{
                consts.DST: '10.0.0.0/8',
                consts.GATEWAY: '1.0.0.1',
                consts.TABLE: 10,
                consts.ONLINK: True
            }, {
                consts.DST: '20.0.0.0/8',
                consts.GATEWAY: '1.0.0.2',
                consts.PREFSRC: '1.2.3.4',
                consts.SCOPE: 'link'
            }, {
                consts.DST: '2001:db8:2::1/128',
                consts.GATEWAY: '2001:db8::1'
            }],
            rules=[{
                consts.SRC: '1.1.1.1',
                consts.SRC_LEN: 32,
                consts.TABLE: 20,
            }, {
                consts.SRC: '2001:db8::1',
                consts.SRC_LEN: 128,
                consts.TABLE: 40,
            }],
            scripts={
                consts.IFACE_UP: [{
                    consts.COMMAND: "post-up eth1"
                }],
                consts.IFACE_DOWN: [{
                    consts.COMMAND: "post-down eth1"
                }],
            })

        idx = mock.MagicMock()
        mock_link_lookup.return_value = [idx]

        mock_get_links.return_value = [{
            consts.STATE: consts.IFACE_DOWN
        }]

        controller = interface.InterfaceController()
        controller.down(iface)

        mock_link.assert_not_called()
        mock_addr.assert_not_called()
        mock_route.assert_not_called()
        mock_rule.assert_not_called()
        mock_check_output.assert_not_called()

    @mock.patch('pyroute2.IPRoute.rule')
    @mock.patch('pyroute2.IPRoute.route')
    @mock.patch('pyroute2.IPRoute.addr')
    @mock.patch('pyroute2.IPRoute.link')
    @mock.patch('pyroute2.IPRoute.get_links')
    @mock.patch('pyroute2.IPRoute.link_lookup')
    @mock.patch('subprocess.check_output')
    def test_down_auto(self, mock_check_output, mock_link_lookup,
                       mock_get_links, mock_link, mock_addr, mock_route,
                       mock_rule):
        iface = interface_file.InterfaceFile(
            name="eth1",
            mtu=1450,
            addresses=[{
                consts.DHCP: True,
                consts.IPV6AUTO: True
            }],
            routes=[],
            rules=[],
            scripts={
                consts.IFACE_UP: [{
                    consts.COMMAND: "post-up eth1"
                }],
                consts.IFACE_DOWN: [{
                    consts.COMMAND: "post-down eth1"
                }],
            })

        idx = mock.MagicMock()
        mock_link_lookup.return_value = [idx]

        mock_get_links.return_value = [{
            consts.STATE: consts.IFACE_UP
        }]

        controller = interface.InterfaceController()
        controller.down(iface)

        mock_link.assert_called_once_with(
            controller.SET,
            index=idx,
            state=consts.IFACE_DOWN)

        mock_addr.assert_not_called()
        mock_route.assert_not_called()
        mock_rule.assert_not_called()

        mock_check_output.assert_has_calls([
            mock.call(["/sbin/dhclient",
                       "-r",
                       "-lf",
                       "/var/lib/dhclient/dhclient-{}.leases".format(
                           iface.name),
                       "-pf",
                       "/run/dhclient-{}.pid".format(iface.name),
                       iface.name], stderr=-2),
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.{}.accept_ra=0".format(iface.name)],
                      stderr=-2),
            mock.call(["/sbin/sysctl", "-w",
                       "net.ipv6.conf.{}.autoconf=0".format(iface.name)],
                      stderr=-2),
            mock.call(["post-down", iface.name])
        ])



#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

"""add l7policy action redirect prefix

Revision ID: 7432f1d4ea83
Revises: 6742ca1b27c2
Create Date: 2018-09-09 20:35:38.780054

"""

from alembic import op
import sqlalchemy as sa

# revision identifiers, used by Alembic.
revision = '7432f1d4ea83'
down_revision = '6742ca1b27c2'


def upgrade():
    op.add_column(
        u'health_monitor',
        sa.Column(u'http_version', sa.Float(), nullable=True)
    )
    op.add_column(
        u'health_monitor',
        sa.Column(u'domain_name', sa.String(255), nullable=True)
    )



# Copyright 2018 Rackspace, US Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import signal
from unittest import mock

from oslo_config import cfg
from oslo_config import fixture as oslo_fixture

import octavia.api.drivers.driver_agent.driver_listener
from octavia.cmd import driver_agent
from octavia.tests.unit import base

CONF = cfg.CONF


class TestDriverAgentCMD(base.TestCase):

    def setUp(self):
        super().setUp()
        self.CONF = self.useFixture(oslo_fixture.Config(cfg.CONF))

    @mock.patch('os.kill')
    @mock.patch('octavia.cmd.driver_agent.CONF')
    def test_handle_mutate_config(self, mock_conf, mock_os_kill):
        driver_agent._handle_mutate_config(1, 2)
        mock_conf.mutate_config_files.assert_called_once()
        os_calls = [mock.call(1, signal.SIGHUP), mock.call(2, signal.SIGHUP)]
        mock_os_kill.assert_has_calls(os_calls, any_order=True)

    def test_check_if_provider_agent_enabled(self):
        mock_extension = mock.MagicMock()
        self.CONF.config(group="driver_agent",
                         enabled_provider_agents=[
                             'spiffy_agent', 'super_agent'])
        mock_extension.name = 'super_agent'
        self.assertTrue(
            driver_agent._check_if_provider_agent_enabled(mock_extension))
        mock_extension.name = 'bogus_agent'
        self.assertFalse(
            driver_agent._check_if_provider_agent_enabled(mock_extension))

    @mock.patch('setproctitle.setproctitle')
    @mock.patch('signal.signal')
    def test_process_wrapper(self, mock_signal, mock_setproctitle):
        mock_exit_event = mock.MagicMock()
        mock_function = mock.MagicMock()
        mock_function.side_effect = [
            mock.DEFAULT, Exception('boom'), mock.DEFAULT, Exception('boom'),
            mock.DEFAULT]
        mock_exit_event.is_set.side_effect = [False, False, True,
                                              False, False, True]

        signal_calls = [mock.call(signal.SIGINT, signal.SIG_IGN),
                        mock.call(signal.SIGHUP, driver_agent._mutate_config)]
        # With agent_name
        driver_agent._process_wrapper(
            mock_exit_event, 'test_proc_name', mock_function,
            agent_name='test_agent_name')
        mock_signal.assert_has_calls(signal_calls)
        mock_setproctitle.assert_called_once_with(
            'octavia-driver-agent - test_proc_name -- test_agent_name')
        mock_function.assert_called_once_with(mock_exit_event)

        # With agent_name - With function exception
        mock_signal.reset_mock()
        mock_setproctitle.reset_mock()
        mock_function.reset_mock()
        driver_agent._process_wrapper(
            mock_exit_event, 'test_proc_name', mock_function,
            agent_name='test_agent_name')
        mock_signal.assert_has_calls(signal_calls)
        mock_setproctitle.assert_called_once_with(
            'octavia-driver-agent - test_proc_name -- test_agent_name')
        mock_function.assert_called_once_with(mock_exit_event)

        # Without agent_name
        mock_signal.reset_mock()
        mock_setproctitle.reset_mock()
        mock_function.reset_mock()
        driver_agent._process_wrapper(
            mock_exit_event, 'test_proc_name', mock_function)
        mock_signal.assert_has_calls(signal_calls)
        mock_setproctitle.assert_called_once_with(
            'octavia-driver-agent - test_proc_name')
        mock_function.assert_called_once_with(mock_exit_event)

        # Without agent_name - With function exception
        mock_signal.reset_mock()
        mock_setproctitle.reset_mock()
        mock_function.reset_mock()
        driver_agent._process_wrapper(
            mock_exit_event, 'test_proc_name', mock_function)
        mock_signal.assert_has_calls(signal_calls)
        mock_setproctitle.assert_called_once_with(
            'octavia-driver-agent - test_proc_name')
        mock_function.assert_called_once_with(mock_exit_event)

    @mock.patch('octavia.cmd.driver_agent.multiprocessing')
    @mock.patch('stevedore.enabled.EnabledExtensionManager')
    def test_start_provider_agents(self, mock_stevedore, mock_multiprocessing):
        mock_extension = mock.MagicMock()
        mock_extension.name = 'test_extension'
        mock_exit_event = mock.MagicMock()
        mock_stevedore.return_value = [mock_extension]
        mock_ext_proc = mock.MagicMock()
        mock_multiprocessing.Process.return_value = mock_ext_proc

        driver_agent._start_provider_agents(mock_exit_event)

        mock_stevedore.assert_called_once_with(
            namespace='octavia.driver_agent.provider_agents',
            check_func=driver_agent._check_if_provider_agent_enabled)
        mock_multiprocessing.Process.assert_called_once_with(
            name='test_extension', target=driver_agent._process_wrapper,
            args=(mock_exit_event, 'provider_agent', mock_extension.plugin),
            kwargs={'agent_name': 'test_extension'})
        mock_ext_proc.start.assert_called_once_with()

    @mock.patch('os.kill')
    @mock.patch('octavia.cmd.driver_agent.multiprocessing')
    @mock.patch('oslo_reports.guru_meditation_report.TextGuruMeditation.'
                'setup_autorun')
    @mock.patch('octavia.common.service.prepare_service')
    def test_main(self, mock_prep_srvc, mock_gmr, mock_multiprocessing,
                  mock_kill):
        mock_exit_event = mock.MagicMock()
        mock_multiprocessing.Event.return_value = mock_exit_event
        mock_status_listener_proc = mock.MagicMock()
        mock_stats_listener_proc = mock.MagicMock()
        mock_get_listener_proc = mock.MagicMock()
        mock_multiprocessing.Process.side_effect = [
            mock_status_listener_proc, mock_stats_listener_proc,
            mock_get_listener_proc,
            mock_status_listener_proc, mock_stats_listener_proc,
            mock_get_listener_proc,
            mock_status_listener_proc, mock_stats_listener_proc,
            mock_get_listener_proc,
            mock_status_listener_proc, mock_stats_listener_proc,
            mock_get_listener_proc,
            mock_status_listener_proc, mock_stats_listener_proc,
            mock_get_listener_proc]
        driver_agent.main()
        mock_prep_srvc.assert_called_once()
        mock_gmr.assert_called_once()
        mock_status_listener_proc.start.assert_called_once()
        mock_stats_listener_proc.start.assert_called_once()
        mock_get_listener_proc.start.assert_called_once()
        process_calls = [mock.call(
            args=mock_exit_event, name='status_listener',
            target=(octavia.api.drivers.driver_agent.driver_listener.
                    status_listener)),
            mock.call(
                args=mock_exit_event, name='stats_listener',
                target=(octavia.api.drivers.driver_agent.driver_listener.
                        stats_listener)),
            mock.call(
                args=mock_exit_event, name='get_listener',
                target=(octavia.api.drivers.driver_agent.driver_listener.
                        get_listener))]
        mock_multiprocessing.Process.has_calls(process_calls, any_order=True)

        # Test keyboard interrupt path
        mock_stats_listener_proc.join.side_effect = [KeyboardInterrupt, None]
        driver_agent.main()
        mock_exit_event.set.assert_called_once()

        # Test keyboard interrupt with provider agents
        mock_exit_event.reset_mock()
        mock_stats_listener_proc.join.side_effect = [KeyboardInterrupt, None]
        mock_provider_proc = mock.MagicMock()
        mock_provider_proc.pid = 'not-valid-pid'
        mock_provider_proc.exitcode = 1
        driver_agent.PROVIDER_AGENT_PROCESSES = [mock_provider_proc]
        driver_agent.main()
        mock_exit_event.set.assert_called_once()
        mock_provider_proc.join.assert_called_once_with(
            CONF.driver_agent.provider_agent_shutdown_timeout)

        # Test keyboard interrupt with provider agents fails to stop
        mock_exit_event.reset_mock()
        mock_stats_listener_proc.join.side_effect = [KeyboardInterrupt, None]
        mock_provider_proc = mock.MagicMock()
        mock_provider_proc.pid = 'not-valid-pid'
        mock_provider_proc.exitcode = None
        driver_agent.PROVIDER_AGENT_PROCESSES = [mock_provider_proc]
        driver_agent.main()
        mock_exit_event.set.assert_called_once()
        mock_provider_proc.join.assert_called_once_with(
            CONF.driver_agent.provider_agent_shutdown_timeout)
        mock_kill.assert_called_once_with('not-valid-pid', signal.SIGKILL)

        # Test keyboard interrupt with provider agents join exception
        mock_exit_event.reset_mock()
        mock_stats_listener_proc.join.side_effect = [KeyboardInterrupt, None]
        mock_provider_proc = mock.MagicMock()
        mock_provider_proc.pid = 'not-valid-pid'
        mock_provider_proc.join.side_effect = Exception('boom')
        driver_agent.PROVIDER_AGENT_PROCESSES = [mock_provider_proc]
        driver_agent.main()
        mock_exit_event.set.assert_called_once()
        mock_provider_proc.join.assert_called_once_with(
            CONF.driver_agent.provider_agent_shutdown_timeout)



#    Copyright 2014 Rackspace
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from wsme import types as wtypes

from octavia.api.common import types


class BaseFlavorProfileType(types.BaseType):
    _type_to_model_map = {}
    _child_map = {}


class FlavorProfileResponse(BaseFlavorProfileType):
    """Defines which attributes are to be shown on any response."""
    id = wtypes.wsattr(wtypes.UuidType())
    name = wtypes.wsattr(wtypes.StringType())
    provider_name = wtypes.wsattr(wtypes.StringType())
    flavor_data = wtypes.wsattr(wtypes.StringType())

    @classmethod
    def from_data_model(cls, data_model, children=False):
        flavorprofile = super(FlavorProfileResponse, cls).from_data_model(
            data_model, children=children)
        return flavorprofile


class FlavorProfileRootResponse(types.BaseType):
    flavorprofile = wtypes.wsattr(FlavorProfileResponse)


class FlavorProfilesRootResponse(types.BaseType):
    flavorprofiles = wtypes.wsattr([FlavorProfileResponse])
    flavorprofile_links = wtypes.wsattr([types.PageType])


class FlavorProfilePOST(BaseFlavorProfileType):
    """Defines mandatory and optional attributes of a POST request."""
    name = wtypes.wsattr(wtypes.StringType(max_length=255), mandatory=True)
    provider_name = wtypes.wsattr(wtypes.StringType(max_length=255),
                                  mandatory=True)
    flavor_data = wtypes.wsattr(wtypes.StringType(max_length=4096),
                                mandatory=True)


class FlavorProfileRootPOST(types.BaseType):
    flavorprofile = wtypes.wsattr(FlavorProfilePOST)


class FlavorProfilePUT(BaseFlavorProfileType):
    """Defines the attributes of a PUT request."""
    name = wtypes.wsattr(wtypes.StringType(max_length=255))
    provider_name = wtypes.wsattr(wtypes.StringType(max_length=255))
    flavor_data = wtypes.wsattr(wtypes.StringType(max_length=4096))


class FlavorProfileRootPUT(types.BaseType):
    flavorprofile = wtypes.wsattr(FlavorProfilePUT)



# Copyright 2014,  Doug Wiegley,  A10 Networks.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import mock

from oslo_utils import uuidutils

from octavia.common import constants
import octavia.common.utils as utils
import octavia.tests.unit.base as base


class TestConfig(base.TestCase):

    def test_get_hostname(self):
        self.assertNotEqual(utils.get_hostname(), '')

    def test_is_ipv4(self):
        self.assertTrue(utils.is_ipv4('192.0.2.10'))
        self.assertTrue(utils.is_ipv4('169.254.0.10'))
        self.assertTrue(utils.is_ipv4('0.0.0.0'))
        self.assertFalse(utils.is_ipv4('::'))
        self.assertFalse(utils.is_ipv4('2001:db8::1'))
        self.assertFalse(utils.is_ipv4('fe80::225:90ff:fefb:53ad'))

    def test_is_ipv6(self):
        self.assertFalse(utils.is_ipv6('192.0.2.10'))
        self.assertFalse(utils.is_ipv6('169.254.0.10'))
        self.assertFalse(utils.is_ipv6('0.0.0.0'))
        self.assertTrue(utils.is_ipv6('::'))
        self.assertTrue(utils.is_ipv6('2001:db8::1'))
        self.assertTrue(utils.is_ipv6('fe80::225:90ff:fefb:53ad'))

    def test_is_ipv6_lla(self):
        self.assertFalse(utils.is_ipv6_lla('192.0.2.10'))
        self.assertFalse(utils.is_ipv6_lla('169.254.0.10'))
        self.assertFalse(utils.is_ipv6_lla('0.0.0.0'))
        self.assertFalse(utils.is_ipv6_lla('::'))
        self.assertFalse(utils.is_ipv6_lla('2001:db8::1'))
        self.assertTrue(utils.is_ipv6_lla('fe80::225:90ff:fefb:53ad'))

    def test_ip_port_str(self):
        self.assertEqual("127.0.0.1:8080",
                         utils.ip_port_str('127.0.0.1', 8080))
        self.assertEqual("[::1]:8080",
                         utils.ip_port_str('::1', 8080))

    def test_netmask_to_prefix(self):
        self.assertEqual(utils.netmask_to_prefix('255.0.0.0'), 8)
        self.assertEqual(utils.netmask_to_prefix('255.255.0.0'), 16)
        self.assertEqual(utils.netmask_to_prefix('255.255.255.0'), 24)
        self.assertEqual(utils.netmask_to_prefix('255.255.255.128'), 25)

    def test_ip_netmask_to_cidr(self):
        self.assertEqual('10.0.0.0/8',
                         utils.ip_netmask_to_cidr('10.0.0.1', '255.0.0.0'))
        self.assertEqual('10.0.0.0/9',
                         utils.ip_netmask_to_cidr('10.0.0.1', '255.128.0.0'))
        self.assertEqual('10.0.0.0/16',
                         utils.ip_netmask_to_cidr('10.0.0.1', '255.255.0.0'))
        self.assertEqual('10.0.0.0/20',
                         utils.ip_netmask_to_cidr('10.0.0.1', '255.255.240.0'))
        self.assertEqual('10.0.0.0/30', utils.ip_netmask_to_cidr(
            '10.0.0.1', '255.255.255.252'))

    def test_expand_expected_codes(self):
        exp_codes = ''
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         set())
        exp_codes = '200'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'200'})
        exp_codes = '200, 201'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'200', '201'})
        exp_codes = '200, 201,202'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'200', '201', '202'})
        exp_codes = '200-202'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'200', '201', '202'})
        exp_codes = '200-202, 205'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'200', '201', '202', '205'})
        exp_codes = '200, 201-203'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'200', '201', '202', '203'})
        exp_codes = '200, 201-203, 205'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'200', '201', '202', '203', '205'})
        exp_codes = '201-200, 205'
        self.assertEqual(utils.expand_expected_codes(exp_codes),
                         {'205'})

    def test_base64_sha1_string(self):
        str_to_sha1 = [
            # no special cases str (no altchars)
            ('77e7d60d-e137-4246-8a84-a25db33571cd',
             'iVZVQ5AKmk2Ae0uGLP0Ue4OseRM='),
            # backward compat amphorae with - in str[1:]
            ('9c6e5f27-a0da-4ceb-afe5-5a81230be42e',
             'NjrNgt3Egl-H5ScbYM5ChtUH3M8='),
            # sha1 would start with -, now replaced with x
            ('4db4a3cf-9fef-4057-b1fd-b2afbf7a8a0f',
             'xxqntK8jJ_gE3QEmh-D1-XgCW_E=')
        ]
        for str, sha1 in str_to_sha1:
            self.assertEqual(sha1, utils.base64_sha1_string(str))

    @mock.patch('stevedore.driver.DriverManager')
    def test_get_amphora_driver(self, mock_stevedore_driver):
        FAKE_AMP_DRIVER = 'fake_amp_drvr'
        driver_mock = mock.MagicMock()
        driver_mock.driver = FAKE_AMP_DRIVER
        mock_stevedore_driver.return_value = driver_mock

        result = utils.get_amphora_driver()

        self.assertEqual(FAKE_AMP_DRIVER, result)

    def test_get_vip_secuirty_group_name(self):
        FAKE_LB_ID = uuidutils.generate_uuid()
        self.assertIsNone(utils.get_vip_security_group_name(None))

        expected_sg_name = constants.VIP_SECURITY_GROUP_PREFIX + FAKE_LB_ID
        self.assertEqual(expected_sg_name,
                         utils.get_vip_security_group_name(FAKE_LB_ID))



# Copyright (c) 2018 NEC, Corp.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import sys

from oslo_config import cfg
from oslo_upgradecheck import common_checks
from oslo_upgradecheck import upgradecheck
from stevedore import driver as stevedore_driver

# Need to import to load config
from octavia.common import config  # noqa: F401 pylint: disable=unused-import
from octavia.common import constants
from octavia.common import policy
from octavia.controller.worker.v2 import taskflow_jobboard_driver as tsk_driver
from octavia.i18n import _

CONF = cfg.CONF


class Checks(upgradecheck.UpgradeCommands):

    """Contains upgrade checks

    Various upgrade checks should be added as separate methods in this class
    and added to _upgrade_checks tuple.
    """

    def _check_persistence(self):
        try:
            pers_driver = tsk_driver.MysqlPersistenceDriver()
            with pers_driver.get_persistence() as pers:
                if pers.engine.dialect.name == 'sqlite':
                    return upgradecheck.Result(
                        upgradecheck.Code.WARNING,
                        _('Persistence database is using sqlite backend. '
                          'Verification required if persistence_connecton URL '
                          'has been set properly.'))
                return pers
        except Exception:
            return upgradecheck.Result(upgradecheck.Code.FAILURE,
                                       _('Failed to connect to persistence '
                                         'backend for AmphoraV2 provider.'))

    def _check_jobboard(self, persistence):
        try:
            jobboard_driver = stevedore_driver.DriverManager(
                namespace='octavia.worker.jobboard_driver',
                name=CONF.task_flow.jobboard_backend_driver,
                invoke_args=(persistence,),
                invoke_on_load=True).driver
            with jobboard_driver.job_board(persistence) as jb:
                if jb.connected:
                    return upgradecheck.Result(
                        upgradecheck.Code.SUCCESS,
                        _('Persistence database and Jobboard backend for '
                          'AmphoraV2 provider configured.'))
        except Exception:
            # Return FAILURE later
            pass

        return upgradecheck.Result(
            upgradecheck.Code.FAILURE,
            _('Failed to connect to jobboard backend for AmphoraV2 provider. '
              'Check jobboard configuration options in task_flow config '
              'section.'))

    def _check_amphorav2(self):
        default_provider_driver = CONF.api_settings.default_provider_driver
        enabled_provider_drivers = CONF.api_settings.enabled_provider_drivers
        if (default_provider_driver == constants.AMPHORAV2 or
                constants.AMPHORAV2 in enabled_provider_drivers):
            persistence = self._check_persistence()
            if isinstance(persistence, upgradecheck.Result):
                return persistence
            return self._check_jobboard(persistence)
        return upgradecheck.Result(upgradecheck.Code.SUCCESS,
                                   _('AmphoraV2 provider is not enabled.'))

    def _check_yaml_policy(self):
        if CONF.oslo_policy.policy_file.lower().endswith('yaml'):
            return upgradecheck.Result(upgradecheck.Code.SUCCESS,
                                       _('The [oslo_policy] policy_file '
                                         'setting is configured for YAML '
                                         'policy file format.'))
        if CONF.oslo_policy.policy_file.lower().endswith('json'):
            return upgradecheck.Result(
                upgradecheck.Code.WARNING,
                _('The [oslo_policy] policy_file setting is configured for '
                  'JSON policy file format. JSON format policy files have '
                  'been deprecated by oslo policy. Please use the oslo policy '
                  'tool to convert your policy file to YAML format. See this '
                  'patch for more information: '
                  'https://review.opendev.org/733650'))
        return upgradecheck.Result(upgradecheck.Code.FAILURE,
                                   _('Unable to determine the [oslo_policy] '
                                     'policy_file setting file format. '
                                     'Please make sure your policy file is '
                                     'in YAML format and has the suffix of '
                                     '.yaml for the filename. Oslo policy '
                                     'has deprecated the JSON file format.'))

    _upgrade_checks = (
        (_('AmphoraV2 Check'), _check_amphorav2),
        (_('YAML Policy File'), _check_yaml_policy),
        (_('Policy File JSON to YAML Migration'),
         (common_checks.check_policy_json, {'conf': CONF})),
    )


def main():
    policy.Policy()
    return upgradecheck.main(
        CONF, project='octavia', upgrade_command=Checks())


if __name__ == '__main__':
    sys.exit(main())



# Copyright 2015 Hewlett Packard Enterprise Development Company LP
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import ipaddress
import os

import jinja2
from oslo_config import cfg

from octavia.amphorae.backends.agent.api_server import util
from octavia.common import constants


KEEPALIVED_TEMPLATE = os.path.abspath(
    os.path.join(os.path.dirname(__file__),
                 'templates/keepalived_base.template'))
CONF = cfg.CONF


class KeepalivedJinjaTemplater(object):

    def __init__(self, keepalived_template=None):
        """Keepalived configuration generation

        :param keepalived_template: Absolute path to keepalived Jinja template
        """
        super().__init__()
        self.keepalived_template = (keepalived_template if
                                    keepalived_template else
                                    KEEPALIVED_TEMPLATE)
        self._jinja_env = None

    def get_template(self, template_file):
        """Returns the specified Jinja configuration template."""
        if not self._jinja_env:
            template_loader = jinja2.FileSystemLoader(
                searchpath=os.path.dirname(template_file))
            self._jinja_env = jinja2.Environment(
                autoescape=True,
                loader=template_loader,
                trim_blocks=True,
                lstrip_blocks=True)
        return self._jinja_env.get_template(os.path.basename(template_file))

    def build_keepalived_config(self, loadbalancer, amphora, vip_cidr):
        """Renders the loadblanacer keepalived configuration for Active/Standby

        :param loadbalancer: A lodabalancer object
        :param amp: An amphora object
        :param vip_cidr: The VIP subnet cidr
        """
        # Note on keepalived configuration: The current base configuration
        # enforced Master election whenever a high priority VRRP instance
        # start advertising its presence. Accordingly, the fallback behavior
        # - which I described in the blueprint - is the default behavior.
        # Although this is a stable behavior, this can be undesirable for
        # several backend services. To disable the fallback behavior, we need
        #  to add the "nopreempt" flag in the backup instance section.
        peers_ips = []

        # Validate the VIP address and see if it is IPv6
        vip = loadbalancer.vip.ip_address
        vip_addr = ipaddress.ip_address(vip)
        vip_ipv6 = vip_addr.version == 6

        # Normalize and validate the VIP subnet CIDR
        vip_network_cidr = None
        if vip_ipv6:
            vip_network_cidr = ipaddress.IPv6Network(vip_cidr).with_prefixlen
        else:
            vip_network_cidr = ipaddress.IPv4Network(vip_cidr).with_prefixlen

        for amp in filter(
            lambda amp: amp.status == constants.AMPHORA_ALLOCATED,
                loadbalancer.amphorae):
            if amp.vrrp_ip != amphora.vrrp_ip:
                peers_ips.append(amp.vrrp_ip)
        return self.get_template(self.keepalived_template).render(
            {'vrrp_group_name': loadbalancer.vrrp_group.vrrp_group_name,
             'amp_role': amphora.role,
             'amp_intf': amphora.vrrp_interface,
             'amp_vrrp_id': amphora.vrrp_id,
             'amp_priority': amphora.vrrp_priority,
             'vrrp_garp_refresh':
                 CONF.keepalived_vrrp.vrrp_garp_refresh_interval,
             'vrrp_garp_refresh_repeat':
                 CONF.keepalived_vrrp.vrrp_garp_refresh_count,
             'vrrp_auth_type': loadbalancer.vrrp_group.vrrp_auth_type,
             'vrrp_auth_pass': loadbalancer.vrrp_group.vrrp_auth_pass,
             'amp_vrrp_ip': amphora.vrrp_ip,
             'peers_vrrp_ips': peers_ips,
             'vip_ip_address': vip,
             'advert_int': loadbalancer.vrrp_group.advert_int,
             'check_script_path': util.keepalived_check_script_path(),
             'vrrp_check_interval':
                 CONF.keepalived_vrrp.vrrp_check_interval,
             'vrrp_fail_count': CONF.keepalived_vrrp.vrrp_fail_count,
             'vrrp_success_count':
                 CONF.keepalived_vrrp.vrrp_success_count,
             'vip_network_cidr': vip_network_cidr,
             'vip_ipv6': vip_ipv6},
            constants=constants)



# Copyright 2017 Redhat.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
import ipaddress
from unittest import mock

from octavia.amphorae.backends.agent.api_server import osutils
from octavia.common import exceptions as octavia_exceptions
from octavia.tests.unit import base


class TestOSUtils(base.TestCase):

    def setUp(self):
        super().setUp()

        self.base_os_util = osutils.BaseOS('unknown')

        with mock.patch('distro.id',
                        return_value='ubuntu'):
            self.ubuntu_os_util = osutils.BaseOS.get_os_util()

        with mock.patch('distro.id',
                        return_value='rhel'):
            self.rh_os_util = osutils.BaseOS.get_os_util()

        with mock.patch('distro.id', return_value='centos'):
            with mock.patch('distro.version', return_value='8'):
                self.centos_os_util = osutils.BaseOS.get_os_util()

        with mock.patch('distro.id', return_value='centos'):
            with mock.patch('distro.version', return_value='7'):
                self.centos7_os_util = osutils.BaseOS.get_os_util()

    def test_get_os_util(self):
        with mock.patch('distro.id',
                        return_value='ubuntu'):
            returned_cls = osutils.BaseOS.get_os_util()
            self.assertIsInstance(returned_cls, osutils.Ubuntu)
        with mock.patch('distro.id',
                        return_value='fedora'):
            returned_cls = osutils.BaseOS.get_os_util()
            self.assertIsInstance(returned_cls, osutils.RH)
        with mock.patch('distro.id',
                        return_value='rhel'):
            returned_cls = osutils.BaseOS.get_os_util()
            self.assertIsInstance(returned_cls, osutils.RH)
        with mock.patch('distro.id',
                        return_value='centos'):
            returned_cls = osutils.BaseOS.get_os_util()
            self.assertIsInstance(returned_cls, osutils.CentOS)
        with mock.patch('distro.id',
                        return_value='FakeOS'):
            self.assertRaises(
                octavia_exceptions.InvalidAmphoraOperatingSystem,
                osutils.BaseOS.get_os_util)

    def test_cmd_get_version_of_installed_package(self):
        package_name = 'foo'
        ubuntu_cmd = "dpkg-query -W -f=${{Version}} {name}".format(
            name=package_name)
        rh_cmd = "rpm -q --queryformat %{{VERSION}} {name}".format(
            name=package_name)

        returned_ubuntu_cmd = (
            self.ubuntu_os_util.cmd_get_version_of_installed_package(
                package_name))
        self.assertEqual(ubuntu_cmd, returned_ubuntu_cmd)

        returned_rh_cmd = (self.rh_os_util.
                           cmd_get_version_of_installed_package(package_name))
        self.assertEqual(rh_cmd, returned_rh_cmd)

    def test_cmd_get_version_of_installed_package_mapped(self):
        package_name = 'haproxy'
        centos7_cmd = "rpm -q --queryformat %{VERSION} haproxy18"

        returned_centos7_cmd = (
            self.centos7_os_util.cmd_get_version_of_installed_package(
                package_name))
        self.assertEqual(centos7_cmd, returned_centos7_cmd)

        centos_cmd = "rpm -q --queryformat %{VERSION} haproxy"
        returned_centos_cmd = (
            self.centos_os_util.cmd_get_version_of_installed_package(
                package_name))
        self.assertEqual(centos_cmd, returned_centos_cmd)

    @mock.patch('octavia.amphorae.backends.utils.interface_file.'
                'VIPInterfaceFile')
    def test_write_vip_interface_file(self, mock_vip_interface_file):
        netns_interface = u'eth1234'
        FIXED_IP = u'192.0.2.2'
        SUBNET_CIDR = u'192.0.2.0/24'
        GATEWAY = u'192.51.100.1'
        DEST1 = u'198.51.100.0/24'
        DEST2 = u'203.0.113.0/24'
        NEXTHOP = u'192.0.2.1'
        MTU = 1450
        FIXED_IP_IPV6 = u'2001:0db8:0000:0000:0000:0000:0000:0001'
        # Subnet prefix is purposefully not 32, because that coincidentally
        # matches the result of any arbitrary IPv4->prefixlen conversion
        SUBNET_CIDR_IPV6 = u'2001:db8::/70'

        ip = ipaddress.ip_address(FIXED_IP)
        network = ipaddress.ip_network(SUBNET_CIDR)

        ipv6 = ipaddress.ip_address(FIXED_IP_IPV6)
        networkv6 = ipaddress.ip_network(SUBNET_CIDR_IPV6)

        host_routes = [
            {'nexthop': NEXTHOP, 'destination': DEST1},
            {'nexthop': NEXTHOP, 'destination': DEST2}
        ]

        self.ubuntu_os_util.write_vip_interface_file(
            interface=netns_interface,
            vip=FIXED_IP,
            ip_version=ip.version,
            prefixlen=network.prefixlen,
            gateway=GATEWAY,
            mtu=MTU,
            vrrp_ip=None,
            host_routes=host_routes)

        mock_vip_interface_file.assert_called_once_with(
            name=netns_interface,
            vip=FIXED_IP,
            ip_version=ip.version,
            prefixlen=network.prefixlen,
            gateway=GATEWAY,
            mtu=MTU,
            vrrp_ip=None,
            host_routes=host_routes,
            topology="SINGLE")
        mock_vip_interface_file.return_value.write.assert_called_once()

        # Now test with an IPv6 VIP
        mock_vip_interface_file.reset_mock()

        self.ubuntu_os_util.write_vip_interface_file(
            interface=netns_interface,
            vip=FIXED_IP_IPV6,
            ip_version=ipv6.version,
            prefixlen=networkv6.prefixlen,
            gateway=GATEWAY,
            mtu=MTU,
            vrrp_ip=None,
            host_routes=host_routes)

        mock_vip_interface_file.assert_called_once_with(
            name=netns_interface,
            vip=FIXED_IP_IPV6,
            ip_version=ipv6.version,
            prefixlen=networkv6.prefixlen,
            gateway=GATEWAY,
            mtu=MTU,
            vrrp_ip=None,
            host_routes=host_routes,
            topology="SINGLE")

    @mock.patch('octavia.amphorae.backends.utils.interface_file.'
                'PortInterfaceFile')
    def test_write_port_interface_file(self, mock_port_interface_file):
        FIXED_IP = u'192.0.2.2'
        NEXTHOP = u'192.0.2.1'
        DEST = u'198.51.100.0/24'
        host_routes = [
            {'nexthop': NEXTHOP, 'destination': ipaddress.ip_network(DEST)}
        ]
        FIXED_IP_IPV6 = u'2001:db8::2'
        NEXTHOP_IPV6 = u'2001:db8::1'
        DEST_IPV6 = u'2001:db8:51:100::/64'
        host_routes_ipv6 = [
            {'nexthop': NEXTHOP_IPV6,
             'destination': ipaddress.ip_network(DEST_IPV6)}
        ]
        ip_addr = {'ip_address': FIXED_IP, 'host_routes': host_routes}
        ipv6_addr = {'ip_address': FIXED_IP_IPV6,
                     'host_routes': host_routes_ipv6}

        netns_interface = 'eth1234'
        MTU = 1450
        fixed_ips = [ip_addr, ipv6_addr]

        self.base_os_util.write_port_interface_file(
            interface=netns_interface,
            fixed_ips=fixed_ips,
            mtu=MTU)

        mock_port_interface_file.assert_called_once_with(
            name=netns_interface,
            fixed_ips=fixed_ips,
            mtu=MTU)
        mock_port_interface_file.return_value.write.assert_called_once()



# Copyright 2016 Blue Box, an IBM Company
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from taskflow.patterns import linear_flow

from octavia.common import constants
from octavia.controller.worker.v1.tasks import amphora_driver_tasks
from octavia.controller.worker.v1.tasks import database_tasks
from octavia.controller.worker.v1.tasks import lifecycle_tasks
from octavia.controller.worker.v1.tasks import model_tasks


class L7RuleFlows(object):

    def get_create_l7rule_flow(self):
        """Create a flow to create an L7 rule

        :returns: The flow for creating an L7 rule
        """
        create_l7rule_flow = linear_flow.Flow(constants.CREATE_L7RULE_FLOW)
        create_l7rule_flow.add(lifecycle_tasks.L7RuleToErrorOnRevertTask(
            requires=[constants.L7RULE,
                      constants.LISTENERS,
                      constants.LOADBALANCER]))
        create_l7rule_flow.add(database_tasks.MarkL7RulePendingCreateInDB(
            requires=constants.L7RULE))
        create_l7rule_flow.add(amphora_driver_tasks.ListenersUpdate(
            requires=constants.LOADBALANCER))
        create_l7rule_flow.add(database_tasks.MarkL7RuleActiveInDB(
            requires=constants.L7RULE))
        create_l7rule_flow.add(database_tasks.MarkL7PolicyActiveInDB(
            requires=constants.L7POLICY))
        create_l7rule_flow.add(database_tasks.MarkLBAndListenersActiveInDB(
            requires=[constants.LOADBALANCER, constants.LISTENERS]))

        return create_l7rule_flow

    def get_delete_l7rule_flow(self):
        """Create a flow to delete an L7 rule

        :returns: The flow for deleting an L7 rule
        """
        delete_l7rule_flow = linear_flow.Flow(constants.DELETE_L7RULE_FLOW)
        delete_l7rule_flow.add(lifecycle_tasks.L7RuleToErrorOnRevertTask(
            requires=[constants.L7RULE,
                      constants.LISTENERS,
                      constants.LOADBALANCER]))
        delete_l7rule_flow.add(database_tasks.MarkL7RulePendingDeleteInDB(
            requires=constants.L7RULE))
        delete_l7rule_flow.add(model_tasks.DeleteModelObject(
            rebind={constants.OBJECT: constants.L7RULE}))
        delete_l7rule_flow.add(amphora_driver_tasks.ListenersUpdate(
            requires=constants.LOADBALANCER))
        delete_l7rule_flow.add(database_tasks.DeleteL7RuleInDB(
            requires=constants.L7RULE))
        delete_l7rule_flow.add(database_tasks.DecrementL7ruleQuota(
            requires=constants.L7RULE))
        delete_l7rule_flow.add(database_tasks.MarkL7PolicyActiveInDB(
            requires=constants.L7POLICY))
        delete_l7rule_flow.add(database_tasks.MarkLBAndListenersActiveInDB(
            requires=[constants.LOADBALANCER, constants.LISTENERS]))

        return delete_l7rule_flow

    def get_update_l7rule_flow(self):
        """Create a flow to update an L7 rule

        :returns: The flow for updating an L7 rule
        """
        update_l7rule_flow = linear_flow.Flow(constants.UPDATE_L7RULE_FLOW)
        update_l7rule_flow.add(lifecycle_tasks.L7RuleToErrorOnRevertTask(
            requires=[constants.L7RULE,
                      constants.LISTENERS,
                      constants.LOADBALANCER]))
        update_l7rule_flow.add(database_tasks.MarkL7RulePendingUpdateInDB(
            requires=constants.L7RULE))
        update_l7rule_flow.add(amphora_driver_tasks.ListenersUpdate(
            requires=constants.LOADBALANCER))
        update_l7rule_flow.add(database_tasks.UpdateL7RuleInDB(
            requires=[constants.L7RULE, constants.UPDATE_DICT]))
        update_l7rule_flow.add(database_tasks.MarkL7RuleActiveInDB(
            requires=constants.L7RULE))
        update_l7rule_flow.add(database_tasks.MarkL7PolicyActiveInDB(
            requires=constants.L7POLICY))
        update_l7rule_flow.add(database_tasks.MarkLBAndListenersActiveInDB(
            requires=[constants.LOADBALANCER, constants.LISTENERS]))

        return update_l7rule_flow



#    Copyright 2014 Rackspace
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from wsme import types as wtypes

from octavia.api.common import types
from octavia.common import constants


class BaseMemberType(types.BaseType):
    _type_to_model_map = {'admin_state_up': 'enabled',
                          'address': 'ip_address'}
    _child_map = {}


class MemberResponse(BaseMemberType):
    """Defines which attributes are to be shown on any response."""
    id = wtypes.wsattr(wtypes.UuidType())
    name = wtypes.wsattr(wtypes.StringType())
    operating_status = wtypes.wsattr(wtypes.StringType())
    provisioning_status = wtypes.wsattr(wtypes.StringType())
    admin_state_up = wtypes.wsattr(bool)
    address = wtypes.wsattr(types.IPAddressType())
    protocol_port = wtypes.wsattr(wtypes.IntegerType())
    weight = wtypes.wsattr(wtypes.IntegerType())
    backup = wtypes.wsattr(bool)
    subnet_id = wtypes.wsattr(wtypes.UuidType())
    project_id = wtypes.wsattr(wtypes.StringType())
    created_at = wtypes.wsattr(wtypes.datetime.datetime)
    updated_at = wtypes.wsattr(wtypes.datetime.datetime)
    monitor_address = wtypes.wsattr(types.IPAddressType())
    monitor_port = wtypes.wsattr(wtypes.IntegerType())
    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType()))

    @classmethod
    def from_data_model(cls, data_model, children=False):
        member = super(MemberResponse, cls).from_data_model(
            data_model, children=children)
        return member


class MemberFullResponse(MemberResponse):
    @classmethod
    def _full_response(cls):
        return True


class MemberRootResponse(types.BaseType):
    member = wtypes.wsattr(MemberResponse)


class MembersRootResponse(types.BaseType):
    members = wtypes.wsattr([MemberResponse])
    members_links = wtypes.wsattr([types.PageType])


class MemberPOST(BaseMemberType):
    """Defines mandatory and optional attributes of a POST request."""
    name = wtypes.wsattr(wtypes.StringType(max_length=255))
    admin_state_up = wtypes.wsattr(bool, default=True)
    address = wtypes.wsattr(types.IPAddressType(), mandatory=True)
    protocol_port = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_PORT_NUMBER, maximum=constants.MAX_PORT_NUMBER),
        mandatory=True)
    weight = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_WEIGHT, maximum=constants.MAX_WEIGHT),
        default=constants.DEFAULT_WEIGHT)
    backup = wtypes.wsattr(bool, default=False)
    subnet_id = wtypes.wsattr(wtypes.UuidType())
    # TODO(johnsom) Remove after deprecation (R series)
    project_id = wtypes.wsattr(wtypes.StringType(max_length=36))
    monitor_port = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_PORT_NUMBER, maximum=constants.MAX_PORT_NUMBER),
        default=None)
    monitor_address = wtypes.wsattr(types.IPAddressType(), default=None)
    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(max_length=255)))


class MemberRootPOST(types.BaseType):
    member = wtypes.wsattr(MemberPOST)


class MemberPUT(BaseMemberType):
    """Defines attributes that are acceptable of a PUT request."""
    name = wtypes.wsattr(wtypes.StringType(max_length=255))
    admin_state_up = wtypes.wsattr(bool)
    weight = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_WEIGHT, maximum=constants.MAX_WEIGHT))
    backup = wtypes.wsattr(bool)
    monitor_port = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_PORT_NUMBER, maximum=constants.MAX_PORT_NUMBER))
    monitor_address = wtypes.wsattr(types.IPAddressType())
    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(max_length=255)))


class MemberRootPUT(types.BaseType):
    member = wtypes.wsattr(MemberPUT)


class MembersRootPUT(types.BaseType):
    members = wtypes.wsattr([MemberPOST])


class MemberSingleCreate(BaseMemberType):
    """Defines mandatory and optional attributes of a POST request."""
    name = wtypes.wsattr(wtypes.StringType(max_length=255))
    admin_state_up = wtypes.wsattr(bool, default=True)
    address = wtypes.wsattr(types.IPAddressType(), mandatory=True)
    protocol_port = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_PORT_NUMBER, maximum=constants.MAX_PORT_NUMBER),
        mandatory=True)
    weight = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_WEIGHT, maximum=constants.MAX_WEIGHT),
        default=constants.DEFAULT_WEIGHT)
    backup = wtypes.wsattr(bool, default=False)
    subnet_id = wtypes.wsattr(wtypes.UuidType())
    monitor_port = wtypes.wsattr(wtypes.IntegerType(
        minimum=constants.MIN_PORT_NUMBER, maximum=constants.MAX_PORT_NUMBER))
    monitor_address = wtypes.wsattr(types.IPAddressType())
    tags = wtypes.wsattr(wtypes.ArrayType(wtypes.StringType(max_length=255)))


class MemberStatusResponse(BaseMemberType):
    """Defines which attributes are to be shown on status response."""
    id = wtypes.wsattr(wtypes.UuidType())
    name = wtypes.wsattr(wtypes.StringType())
    operating_status = wtypes.wsattr(wtypes.StringType())
    provisioning_status = wtypes.wsattr(wtypes.StringType())
    address = wtypes.wsattr(types.IPAddressType())
    protocol_port = wtypes.wsattr(wtypes.IntegerType())

    @classmethod
    def from_data_model(cls, data_model, children=False):
        member = super(MemberStatusResponse, cls).from_data_model(
            data_model, children=children)

        if not member.name:
            member.name = ""

        return member



# Copyright 2020 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import socket
import struct
import sys
import time

import random


# Adapted from https://opendev.org/openstack/os-ken/src/branch/
#   master/os_ken/lib/packet/sctp.py
def crc32c(data):
    # from RFC 3309
    crc_c = [
        0x00000000, 0xF26B8303, 0xE13B70F7, 0x1350F3F4,
        0xC79A971F, 0x35F1141C, 0x26A1E7E8, 0xD4CA64EB,
        0x8AD958CF, 0x78B2DBCC, 0x6BE22838, 0x9989AB3B,
        0x4D43CFD0, 0xBF284CD3, 0xAC78BF27, 0x5E133C24,
        0x105EC76F, 0xE235446C, 0xF165B798, 0x030E349B,
        0xD7C45070, 0x25AFD373, 0x36FF2087, 0xC494A384,
        0x9A879FA0, 0x68EC1CA3, 0x7BBCEF57, 0x89D76C54,
        0x5D1D08BF, 0xAF768BBC, 0xBC267848, 0x4E4DFB4B,
        0x20BD8EDE, 0xD2D60DDD, 0xC186FE29, 0x33ED7D2A,
        0xE72719C1, 0x154C9AC2, 0x061C6936, 0xF477EA35,
        0xAA64D611, 0x580F5512, 0x4B5FA6E6, 0xB93425E5,
        0x6DFE410E, 0x9F95C20D, 0x8CC531F9, 0x7EAEB2FA,
        0x30E349B1, 0xC288CAB2, 0xD1D83946, 0x23B3BA45,
        0xF779DEAE, 0x05125DAD, 0x1642AE59, 0xE4292D5A,
        0xBA3A117E, 0x4851927D, 0x5B016189, 0xA96AE28A,
        0x7DA08661, 0x8FCB0562, 0x9C9BF696, 0x6EF07595,
        0x417B1DBC, 0xB3109EBF, 0xA0406D4B, 0x522BEE48,
        0x86E18AA3, 0x748A09A0, 0x67DAFA54, 0x95B17957,
        0xCBA24573, 0x39C9C670, 0x2A993584, 0xD8F2B687,
        0x0C38D26C, 0xFE53516F, 0xED03A29B, 0x1F682198,
        0x5125DAD3, 0xA34E59D0, 0xB01EAA24, 0x42752927,
        0x96BF4DCC, 0x64D4CECF, 0x77843D3B, 0x85EFBE38,
        0xDBFC821C, 0x2997011F, 0x3AC7F2EB, 0xC8AC71E8,
        0x1C661503, 0xEE0D9600, 0xFD5D65F4, 0x0F36E6F7,
        0x61C69362, 0x93AD1061, 0x80FDE395, 0x72966096,
        0xA65C047D, 0x5437877E, 0x4767748A, 0xB50CF789,
        0xEB1FCBAD, 0x197448AE, 0x0A24BB5A, 0xF84F3859,
        0x2C855CB2, 0xDEEEDFB1, 0xCDBE2C45, 0x3FD5AF46,
        0x7198540D, 0x83F3D70E, 0x90A324FA, 0x62C8A7F9,
        0xB602C312, 0x44694011, 0x5739B3E5, 0xA55230E6,
        0xFB410CC2, 0x092A8FC1, 0x1A7A7C35, 0xE811FF36,
        0x3CDB9BDD, 0xCEB018DE, 0xDDE0EB2A, 0x2F8B6829,
        0x82F63B78, 0x709DB87B, 0x63CD4B8F, 0x91A6C88C,
        0x456CAC67, 0xB7072F64, 0xA457DC90, 0x563C5F93,
        0x082F63B7, 0xFA44E0B4, 0xE9141340, 0x1B7F9043,
        0xCFB5F4A8, 0x3DDE77AB, 0x2E8E845F, 0xDCE5075C,
        0x92A8FC17, 0x60C37F14, 0x73938CE0, 0x81F80FE3,
        0x55326B08, 0xA759E80B, 0xB4091BFF, 0x466298FC,
        0x1871A4D8, 0xEA1A27DB, 0xF94AD42F, 0x0B21572C,
        0xDFEB33C7, 0x2D80B0C4, 0x3ED04330, 0xCCBBC033,
        0xA24BB5A6, 0x502036A5, 0x4370C551, 0xB11B4652,
        0x65D122B9, 0x97BAA1BA, 0x84EA524E, 0x7681D14D,
        0x2892ED69, 0xDAF96E6A, 0xC9A99D9E, 0x3BC21E9D,
        0xEF087A76, 0x1D63F975, 0x0E330A81, 0xFC588982,
        0xB21572C9, 0x407EF1CA, 0x532E023E, 0xA145813D,
        0x758FE5D6, 0x87E466D5, 0x94B49521, 0x66DF1622,
        0x38CC2A06, 0xCAA7A905, 0xD9F75AF1, 0x2B9CD9F2,
        0xFF56BD19, 0x0D3D3E1A, 0x1E6DCDEE, 0xEC064EED,
        0xC38D26C4, 0x31E6A5C7, 0x22B65633, 0xD0DDD530,
        0x0417B1DB, 0xF67C32D8, 0xE52CC12C, 0x1747422F,
        0x49547E0B, 0xBB3FFD08, 0xA86F0EFC, 0x5A048DFF,
        0x8ECEE914, 0x7CA56A17, 0x6FF599E3, 0x9D9E1AE0,
        0xD3D3E1AB, 0x21B862A8, 0x32E8915C, 0xC083125F,
        0x144976B4, 0xE622F5B7, 0xF5720643, 0x07198540,
        0x590AB964, 0xAB613A67, 0xB831C993, 0x4A5A4A90,
        0x9E902E7B, 0x6CFBAD78, 0x7FAB5E8C, 0x8DC0DD8F,
        0xE330A81A, 0x115B2B19, 0x020BD8ED, 0xF0605BEE,
        0x24AA3F05, 0xD6C1BC06, 0xC5914FF2, 0x37FACCF1,
        0x69E9F0D5, 0x9B8273D6, 0x88D28022, 0x7AB90321,
        0xAE7367CA, 0x5C18E4C9, 0x4F48173D, 0xBD23943E,
        0xF36E6F75, 0x0105EC76, 0x12551F82, 0xE03E9C81,
        0x34F4F86A, 0xC69F7B69, 0xD5CF889D, 0x27A40B9E,
        0x79B737BA, 0x8BDCB4B9, 0x988C474D, 0x6AE7C44E,
        0xBE2DA0A5, 0x4C4623A6, 0x5F16D052, 0xAD7D5351,
    ]

    crc32 = 0xffffffff
    for c in data:
        crc32 = (crc32 >> 8) ^ crc_c[(crc32 ^ (c)) & 0xFF]
    crc32 = (~crc32) & 0xffffffff
    return struct.unpack(">I", struct.pack("<I", crc32))[0]


def _sctp_build_init_packet(src_port, dest_port, tag):
    sctp_hdr = [
        # HEADER
        struct.pack('!H', src_port),      # Source port number
        struct.pack('!H', dest_port),     # Destination port number
        struct.pack('!L', 0),             # Verification tag
        struct.pack('!L', 0),             # checksum

        # INIT Chunk
        struct.pack('!B', 1),             # Type
        struct.pack('!B', 0),             # Chunk flag
        struct.pack('!H', 0),             # Chunk length
        struct.pack('!L', tag),           # Tag
        struct.pack('!L', 106496),        # a_rwnd
        struct.pack('!H', 10),            # Number of outbound stream
        struct.pack('!H', 65535),         # Number of inbound stream
        struct.pack('!L',
                    random.randint(
                        1, 4294967295)),  # Initial TSN
    ]

    data = bytearray(b''.join(sctp_hdr))

    chunk_len = len(data) - 12
    struct.pack_into('!H', data, 14, chunk_len)

    checksum = crc32c(data)
    struct.pack_into('!L', data, 8, checksum)

    return data


def _sctp_build_abort_packet(src_port, dest_port, verification_tag):
    sctp_hdr = [
        # HEADER
        struct.pack('!H', src_port),          # Source port number
        struct.pack('!H', dest_port),         # Destination port number
        struct.pack('!L', verification_tag),  # Verification tag
        struct.pack('!L', 0),                 # checksum

        # ABORT Chunk
        struct.pack('!B', 6),                 # Type
        struct.pack('!B', 1),                 # Chunk flag
        struct.pack('!H', 4),                 # Chunk length
    ]

    data = bytearray(b''.join(sctp_hdr))

    checksum = crc32c(data)
    struct.pack_into('!L', data, 8, checksum)

    return data


def _sctp_decode_packet(data, family, expected_tag):
    # AF_INET packets contain ipv4 header
    if family == socket.AF_INET:
        hdr_offset = (data[0] & 0xf) << 2
    else:
        hdr_offset = 0

    if len(data) - hdr_offset < 16:
        return False

    # Check if the packet is a reply to our INIT packet
    verification_tag = struct.unpack_from('!L', data, hdr_offset + 4)[0]
    if verification_tag != expected_tag:
        return False

    response_type = data[hdr_offset + 12]

    return response_type


def sctp_health_check(ip_address, port, timeout=2):
    family = socket.AF_INET6 if ':' in ip_address else socket.AF_INET

    s = socket.socket(family, socket.SOCK_RAW, socket.IPPROTO_SCTP)

    s.settimeout(.2)

    s.connect((ip_address, port))

    tag = random.randint(1, 4294967295)
    src_port = random.randint(1024, 65535)

    data = _sctp_build_init_packet(src_port, port, tag)

    print("Sending INIT packet to {}:{}".format(ip_address, port))
    s.send(data)

    start = time.time()

    send_abort = False

    while time.time() - start < timeout:
        try:
            buf, addr = s.recvfrom(1500)
        except socket.timeout:
            continue

        response_type = _sctp_decode_packet(buf, family, tag)
        if response_type is False:
            continue

        if response_type == 2:  # INIT ACK
            print("Received INIT ACK")
            send_abort = True
            ret = 0
        elif response_type == 6:  # ABORT
            print("Received ABORT")
            ret = 1
        else:  # Others: unknown error
            print("Received {} Type chunk".format(response_type))
            send_abort = True
            ret = 3

        break
    else:
        print("Timeout after {} seconds.".format(timeout))
        # Timeout
        ret = 2

    if send_abort:
        has_sctp_support = False

        with open("/proc/net/protocols") as fp:
            for line in fp:
                if line.startswith('SCTP'):
                    has_sctp_support = True

        # if SCTP support is not included in the kernel, closing the socket
        # won't automatically send a ABORT packet, we need to craft it.
        if not has_sctp_support:
            data = _sctp_build_abort_packet(src_port, port, tag)

            print("Sending ABORT packet")
            s.send(data)

    s.close()

    return ret


def main():
    default_timeout = 3

    args = sys.argv[1:]

    protocol = args.pop(0)

    if args[0] == '-t':
        args.pop(0)
        default_timeout = int(args.pop(0))

    if protocol.lower() == 'sctp':
        destination = args[0]
        port = int(args[1])

        ret = sctp_health_check(destination, port, timeout=default_timeout)
        sys.exit(ret)
    else:
        print("Unsupported protocol '{}'".format(protocol))
        sys.exit(1)


if __name__ == '__main__':
    main()



# Copyright 2015 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#

from taskflow import task


class DeleteModelObject(task.Task):
    """Task to delete an object in a model."""

    def execute(self, object):

        object.delete()


class UpdateAttributes(task.Task):
    """Task to update an object for changes."""

    def execute(self, object, update_dict):
        """Update an object and its associated resources.

        Note: This relies on the data_model update() methods to handle complex
              objects with nested objects (LoadBalancer.vip,
              Pool.session_persistence, etc.)

        :param object: The object will be updated.
        :param update_dict: The updates dictionary.
        :returns: None
        """
        object.update(update_dict)



# Copyright 2020 Red Hat, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from taskflow import retry

from octavia.controller.worker.v2.tasks import retry_tasks
import octavia.tests.unit.base as base


class TestRetryTasks(base.TestCase):

    def setUp(self):
        super(TestRetryTasks, self).setUp()

    @mock.patch('time.sleep')
    def test_sleeping_retry_times_controller(self, mock_sleep):
        retry_ctrlr = retry_tasks.SleepingRetryTimesController(
            attempts=2, name='test_retry')

        # Test on_failure that should RETRY
        history = ['boom']

        result = retry_ctrlr.on_failure(history)

        self.assertEqual(retry.RETRY, result)

        # Test on_failure retries exhausted, should REVERT
        history = ['boom', 'bang', 'pow']

        result = retry_ctrlr.on_failure(history)

        self.assertEqual(retry.REVERT, result)

        # Test revert - should not raise
        retry_ctrlr.revert(history)



# Copyright 2018 Rackspace US Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import copy
import datetime
import random

from oslo_utils import uuidutils

from octavia.common import constants
from octavia.common import data_models
import octavia.tests.unit.base as base


class TestDataModels(base.TestCase):

    def setUp(self):

        self.LB_ID = uuidutils.generate_uuid()
        self.LISTENER_ID = uuidutils.generate_uuid()
        self.PROJECT_ID = uuidutils.generate_uuid()
        self.SERVER_GROUP_ID = uuidutils.generate_uuid()
        self.CREATED_AT = datetime.datetime.now()
        self.UPDATED_AT = datetime.datetime.utcnow()
        self.VIP_IP = '192.0.2.10'
        self.VIP_SUBNET_ID = uuidutils.generate_uuid()
        self.VIP_NETWORK_ID = uuidutils.generate_uuid()
        self.VIP_PORT_ID = uuidutils.generate_uuid()
        self.VIP_QOS_ID = uuidutils.generate_uuid()
        self.POOL_ID = uuidutils.generate_uuid()
        self.AMP_ID = uuidutils.generate_uuid()
        self.COMPUTE_ID = uuidutils.generate_uuid()
        self.IMAGE_ID = uuidutils.generate_uuid()
        self.COMPUTE_FLAVOR = uuidutils.generate_uuid()

        self.LB_obj = data_models.LoadBalancer(
            id=self.LB_ID,
            project_id=self.PROJECT_ID,
            name='test-lb',
            description='test-lb-description',
            provisioning_status='great',
            operating_status='even-better',
            enabled=True,
            vip=None,
            vrrp_group=1,
            topology='infinite',
            listeners=[],
            amphorae=[],
            pools=[],
            server_group_id=self.SERVER_GROUP_ID,
            created_at=self.CREATED_AT,
            updated_at=self.UPDATED_AT)

        self.VIP_obj = data_models.Vip(
            load_balancer_id=self.LB_ID,
            ip_address=self.VIP_IP,
            subnet_id=self.VIP_SUBNET_ID,
            network_id=self.VIP_NETWORK_ID,
            port_id=self.VIP_PORT_ID,
            qos_policy_id=self.VIP_QOS_ID)

        self.POOL_obj = data_models.Pool(
            id=self.POOL_ID,
            project_id=self.PROJECT_ID,
            name='test-pool',
            description='test-pool-description',
            load_balancer_id=self.LB_ID,
            load_balancer=None,
            protocol='avian',
            lb_algorithm='UseAllofThem',
            enabled=True,
            provisioning_status='great',
            operating_status='even-better',
            members=[],
            health_monitor=None,
            session_persistence=None,
            listeners=[],
            l7policies=[],
            created_at=self.CREATED_AT,
            updated_at=self.UPDATED_AT)

        self.SP_obj = data_models.SessionPersistence(
            pool_id=self.POOL_ID,
            type='adhesive',
            cookie_name='chocolate',
            pool=None)

        self.AMP_obj = data_models.Amphora(
            id=self.AMP_ID,
            load_balancer_id=self.LB_ID,
            compute_id=self.COMPUTE_ID,
            status=constants.ACTIVE,
            lb_network_ip=None,
            vrrp_ip=None,
            ha_ip=None,
            vrrp_port_id=None,
            ha_port_id=self.VIP_PORT_ID,
            load_balancer=self.LB_obj,
            role=constants.ROLE_MASTER,
            cert_expiration=None,
            cert_busy=False,
            vrrp_interface=None,
            vrrp_id=None,
            vrrp_priority=constants.ROLE_MASTER_PRIORITY,
            cached_zone=None,
            created_at=self.CREATED_AT,
            updated_at=self.UPDATED_AT,
            image_id=self.IMAGE_ID,
            compute_flavor=self.COMPUTE_FLAVOR
        )

        self.QUOTA_obj = data_models.Quotas(
            project_id=self.PROJECT_ID,
            load_balancer=None,
            listener=None,
            pool=None,
            health_monitor=None,
            member=None,
            l7policy=None,
            l7rule=None,
            in_use_health_monitor=None,
            in_use_listener=None,
            in_use_load_balancer=None,
            in_use_member=None,
            in_use_pool=None,
            in_use_l7policy=None,
            in_use_l7rule=None
        )

        super().setUp()

    def test_LoadBalancer_update(self):

        new_id = uuidutils.generate_uuid()
        new_project_id = uuidutils.generate_uuid()
        new_server_group_id = uuidutils.generate_uuid()
        new_created_at = self.CREATED_AT + datetime.timedelta(minutes=5)
        new_updated_at = self.UPDATED_AT + datetime.timedelta(minutes=10)
        new_name = 'new-test-lb'
        new_description = 'new-test-lb-description'
        new_provisioning_status = 'new-great'
        new_operating_status = 'new-even-better'
        new_enabled = False
        new_vrrp_group = 2
        new_topology = 'new-infinite'

        reference_LB_obj = data_models.LoadBalancer(
            id=new_id,
            project_id=new_project_id,
            name=new_name,
            description=new_description,
            provisioning_status=new_provisioning_status,
            operating_status=new_operating_status,
            enabled=new_enabled,
            vip=None,
            vrrp_group=new_vrrp_group,
            topology=new_topology,
            listeners=[],
            amphorae=[],
            pools=[],
            server_group_id=new_server_group_id,
            created_at=new_created_at,
            updated_at=new_updated_at)

        update_dict = {
            'id': new_id,
            'project_id': new_project_id,
            'name': new_name,
            'description': new_description,
            'provisioning_status': new_provisioning_status,
            'operating_status': new_operating_status,
            'enabled': new_enabled,
            'vrrp_group': new_vrrp_group,
            'topology': new_topology,
            'server_group_id': new_server_group_id,
            'created_at': new_created_at,
            'updated_at': new_updated_at
        }

        test_LB_obj = copy.deepcopy(self.LB_obj)

        test_LB_obj.update(update_dict)

        self.assertEqual(reference_LB_obj, test_LB_obj)

    def test_LoadBalancer_update_add_vip(self):

        new_ip = '192.0.2.44'
        new_subnet_id = uuidutils.generate_uuid()
        new_network_id = uuidutils.generate_uuid()
        new_port_id = uuidutils.generate_uuid()
        new_qos_id = uuidutils.generate_uuid()

        reference_VIP_obj = data_models.Vip(
            load_balancer_id=self.LB_ID,
            ip_address=new_ip,
            subnet_id=new_subnet_id,
            network_id=new_network_id,
            port_id=new_port_id,
            load_balancer=None,
            qos_policy_id=new_qos_id
        )

        update_dict = {
            'vip': {
                'ip_address': new_ip,
                'subnet_id': new_subnet_id,
                'network_id': new_network_id,
                'port_id': new_port_id,
                'load_balancer': None,
                'qos_policy_id': new_qos_id
            }
        }

        test_LB_obj = copy.deepcopy(self.LB_obj)

        test_LB_obj.update(update_dict)

        self.assertEqual(reference_VIP_obj, test_LB_obj.vip)

    def test_LoadBalancer_update_vip_update(self):

        new_id = uuidutils.generate_uuid()
        new_ip = '192.0.2.44'
        new_subnet_id = uuidutils.generate_uuid()
        new_network_id = uuidutils.generate_uuid()
        new_port_id = uuidutils.generate_uuid()
        new_qos_id = uuidutils.generate_uuid()

        reference_VIP_obj = data_models.Vip(
            load_balancer_id=new_id,
            ip_address=new_ip,
            subnet_id=new_subnet_id,
            network_id=new_network_id,
            port_id=new_port_id,
            qos_policy_id=new_qos_id
        )

        update_dict = {
            'vip': {
                'load_balancer_id': new_id,
                'ip_address': new_ip,
                'subnet_id': new_subnet_id,
                'network_id': new_network_id,
                'port_id': new_port_id,
                'qos_policy_id': new_qos_id
            }
        }

        test_LB_obj = copy.deepcopy(self.LB_obj)

        test_LB_obj.vip = copy.deepcopy(self.VIP_obj)

        test_LB_obj.update(update_dict)

        self.assertEqual(reference_VIP_obj, test_LB_obj.vip)

    def test_Pool_update(self):

        new_id = uuidutils.generate_uuid()
        new_project_id = uuidutils.generate_uuid()
        new_name = 'new-test-pool'
        new_description = 'new-test-pool-description'
        new_lb_id = uuidutils.generate_uuid()
        new_protocol = 'sneaker'
        new_lb_algorithm = 'JustOne'
        new_enabled = False
        new_provisioning_status = 'new-great'
        new_operating_status = 'new-even-better'
        new_created_at = self.CREATED_AT + datetime.timedelta(minutes=5)
        new_updated_at = self.UPDATED_AT + datetime.timedelta(minutes=10)

        reference_Pool_obj = data_models.Pool(
            id=new_id,
            project_id=new_project_id,
            name=new_name,
            description=new_description,
            load_balancer_id=new_lb_id,
            protocol=new_protocol,
            lb_algorithm=new_lb_algorithm,
            enabled=new_enabled,
            provisioning_status=new_provisioning_status,
            operating_status=new_operating_status,
            members=[],
            health_monitor=None,
            session_persistence=None,
            listeners=[],
            l7policies=[],
            created_at=new_created_at,
            updated_at=new_updated_at)

        update_dict = {
            'id': new_id,
            'project_id': new_project_id,
            'name': new_name,
            'description': new_description,
            'load_balancer_id': new_lb_id,
            'protocol': new_protocol,
            'lb_algorithm': new_lb_algorithm,
            'enabled': new_enabled,
            'provisioning_status': new_provisioning_status,
            'operating_status': new_operating_status,
            'created_at': new_created_at,
            'updated_at': new_updated_at}

        test_Pool_obj = copy.deepcopy(self.POOL_obj)

        test_Pool_obj.update(update_dict)

        self.assertEqual(reference_Pool_obj, test_Pool_obj)

    def test_Pool_update_add_SP(self):

        new_type = 'glue'
        new_cookie_name = 'chip'

        reference_SP_obj = data_models.SessionPersistence(
            pool_id=self.POOL_ID,
            type=new_type,
            cookie_name=new_cookie_name,
            pool=None)

        update_dict = {
            'session_persistence': {
                'type': new_type,
                'cookie_name': new_cookie_name
            }
        }

        test_Pool_obj = copy.deepcopy(self.POOL_obj)

        test_Pool_obj.update(update_dict)

        self.assertEqual(reference_SP_obj, test_Pool_obj.session_persistence)

    def test_Pool_update_delete_SP(self):

        update_dict = {'session_persistence': {}}

        test_Pool_obj = copy.deepcopy(self.POOL_obj)

        test_Pool_obj.session_persistence = copy.deepcopy(self.SP_obj)

        test_Pool_obj.session_persistence.pool = test_Pool_obj

        test_Pool_obj.update(update_dict)

        self.assertIsNone(test_Pool_obj.session_persistence)

    def test_Pool_update_SP_update(self):

        new_type = 'glue'
        new_cookie_name = 'chip'

        update_dict = {
            'session_persistence': {
                'type': new_type,
                'cookie_name': new_cookie_name
            }
        }

        test_Pool_obj = copy.deepcopy(self.POOL_obj)

        reference_SP_obj = data_models.SessionPersistence(
            pool_id=self.POOL_ID,
            type=new_type,
            cookie_name=new_cookie_name,
            pool=test_Pool_obj)

        test_Pool_obj.session_persistence = copy.deepcopy(self.SP_obj)

        test_Pool_obj.session_persistence.pool = test_Pool_obj

        test_Pool_obj.update(update_dict)

        self.assertEqual(reference_SP_obj, test_Pool_obj.session_persistence)

    def test_Amphora_update(self):

        new_id = uuidutils.generate_uuid()
        new_status = constants.ERROR
        new_role = constants.ROLE_BACKUP
        new_vrrp_priority = constants.ROLE_BACKUP_PRIORITY
        new_created_at = self.CREATED_AT + datetime.timedelta(minutes=5)
        new_updated_at = self.UPDATED_AT + datetime.timedelta(minutes=10)
        new_image_id = uuidutils.generate_uuid()
        new_compute_flavor = uuidutils.generate_uuid()

        update_dict = {
            'id': new_id,
            'status': new_status,
            'role': new_role,
            'vrrp_priority': new_vrrp_priority,
            'created_at': new_created_at,
            'updated_at': new_updated_at,
            'image_id': new_image_id,
            'compute_flavor': new_compute_flavor
        }

        test_Amp_obj = copy.deepcopy(self.AMP_obj)

        reference_Amp_obj = data_models.Amphora(
            id=new_id,
            load_balancer_id=self.LB_ID,
            compute_id=self.COMPUTE_ID,
            status=new_status,
            lb_network_ip=None,
            vrrp_ip=None,
            ha_ip=None,
            vrrp_port_id=None,
            ha_port_id=self.VIP_PORT_ID,
            load_balancer=self.LB_obj,
            role=new_role,
            cert_expiration=None,
            cert_busy=False,
            vrrp_interface=None,
            vrrp_id=None,
            vrrp_priority=constants.ROLE_BACKUP_PRIORITY,
            cached_zone=None,
            created_at=new_created_at,
            updated_at=new_updated_at,
            image_id=new_image_id,
            compute_flavor=new_compute_flavor
        )

        test_Amp_obj.update(update_dict)

        self.assertEqual(reference_Amp_obj, test_Amp_obj)

    def test_Quota_update(self):

        new_loadbalancer_quota = 10
        new_listener_quota = 11
        new_pool_quota = 12
        new_healthmonitor_quota = 13
        new_member_quota = 14
        new_l7policy_quota = 15
        new_l7rule_quota = 16

        update_dict = {
            'load_balancer': new_loadbalancer_quota,
            'listener': new_listener_quota,
            'pool': new_pool_quota,
            'health_monitor': new_healthmonitor_quota,
            'member': new_member_quota,
            'l7policy': new_l7policy_quota,
            'l7rule': new_l7rule_quota
        }

        test_Quota_obj = copy.deepcopy(self.QUOTA_obj)

        reference_Quota_obj = data_models.Quotas(
            project_id=self.PROJECT_ID,
            load_balancer=new_loadbalancer_quota,
            listener=new_listener_quota,
            pool=new_pool_quota,
            health_monitor=new_healthmonitor_quota,
            member=new_member_quota,
            l7policy=new_l7policy_quota,
            l7rule=new_l7rule_quota,
            in_use_health_monitor=None,
            in_use_listener=None,
            in_use_load_balancer=None,
            in_use_member=None,
            in_use_pool=None,
            in_use_l7policy=None,
            in_use_l7rule=None
        )

        test_Quota_obj.update(update_dict)

        self.assertEqual(reference_Quota_obj, test_Quota_obj)

    def test_ListenerStatistics_iadd(self):
        # test incrementing add function

        bytes_in1 = random.randrange(1000000000)
        bytes_out1 = random.randrange(1000000000)
        active_conns1 = random.randrange(1000000000)
        total_conns1 = random.randrange(1000000000)
        request_errors1 = random.randrange(1000000000)
        stats_1 = data_models.ListenerStatistics(
            listener_id=self.LISTENER_ID,
            amphora_id=self.AMP_ID,
            bytes_in=bytes_in1,
            bytes_out=bytes_out1,
            active_connections=active_conns1,
            total_connections=total_conns1,
            request_errors=request_errors1
        )

        bytes_in2 = random.randrange(1000000000)
        bytes_out2 = random.randrange(1000000000)
        active_conns2 = random.randrange(1000000000)
        total_conns2 = random.randrange(1000000000)
        request_errors2 = random.randrange(1000000000)
        stats_2 = data_models.ListenerStatistics(
            listener_id="listener 2",
            amphora_id="amphora 2",
            bytes_in=bytes_in2,
            bytes_out=bytes_out2,
            active_connections=active_conns2,
            total_connections=total_conns2,
            request_errors=request_errors2
        )

        # test successful +=
        stats_1 += stats_2

        # not a delta, so it won't be incremented
        self.assertEqual(stats_1.active_connections, active_conns1)
        self.assertEqual(stats_1.listener_id, self.LISTENER_ID)
        self.assertEqual(stats_1.amphora_id, self.AMP_ID)

        # deltas will be incremented
        self.assertEqual(stats_1.bytes_in, bytes_in1 + bytes_in2)
        self.assertEqual(stats_1.bytes_out, bytes_out1 + bytes_out2)
        self.assertEqual(stats_1.total_connections,
                         total_conns1 + total_conns2)
        self.assertEqual(stats_1.request_errors,
                         request_errors1 + request_errors2)

        # test incrementing an incompatible object
        self.assertRaises(TypeError, stats_1.__iadd__, "boom")



# Copyright 2014 Rackspace
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from unittest import mock

from barbicanclient.v1 import acls
from oslo_config import cfg
from oslo_config import fixture as oslo_fixture


import octavia.certificates.common.auth.barbican_acl as barbican_acl
import octavia.certificates.manager.barbican as barbican_cert_mgr
from octavia.common import keystone
import octavia.tests.unit.base as base

CONF = cfg.CONF


class TestBarbicanACLAuth(base.TestCase):

    def setUp(self):
        super().setUp()
        # Reset the client
        keystone._SESSION = None
        self.conf = self.useFixture(oslo_fixture.Config(cfg.CONF))
        self.conf.config(group="certificates", region_name='RegionOne')
        self.conf.config(group="certificates", endpoint_type='publicURL')

    @mock.patch('keystoneauth1.session.Session', mock.Mock())
    def test_get_barbican_client(self):
        # Mock out the keystone session and get the client
        acl_auth_object = barbican_acl.BarbicanACLAuth()
        bc1 = acl_auth_object.get_barbican_client()

        # Our returned object should have elements that proves it is a real
        # Barbican client object. We shouldn't use `isinstance` because that's
        # an evil pattern, instead we can check for very unique things in the
        # stable client API like "register_consumer", since this should fairly
        # reliably prove we're dealing with a Barbican client.
        self.assertTrue(hasattr(bc1, 'containers') and
                        hasattr(bc1.containers, 'register_consumer'))

        # Getting the session again with new class should get the same object
        acl_auth_object2 = barbican_acl.BarbicanACLAuth()
        bc2 = acl_auth_object2.get_barbican_client()
        self.assertIs(bc1, bc2)

    def test_load_auth_driver(self):
        bcm = barbican_cert_mgr.BarbicanCertManager()
        self.assertIsInstance(bcm.auth, barbican_acl.BarbicanACLAuth)

    @mock.patch('barbicanclient.v1.acls.ACLManager.get')
    @mock.patch('octavia.common.keystone.KeystoneSession')
    def test_ensure_secret_access(self, mock_ksession, mock_aclm):
        acl = mock.MagicMock(spec=acls.SecretACL)
        mock_aclm.return_value = acl

        acl_auth_object = barbican_acl.BarbicanACLAuth()
        acl_auth_object.ensure_secret_access(mock.Mock(), mock.Mock())
        acl.submit.assert_called_once()

    @mock.patch('barbicanclient.v1.acls.ACLManager.get')
    @mock.patch('octavia.common.keystone.KeystoneSession')
    def test_revoke_secret_access(self, mock_ksession, mock_aclm):
        service_user_id = 'uuid1'

        mock_ksession().get_service_user_id.return_value = service_user_id
        acl = mock.MagicMock(spec=acls.SecretACL)
        poacl = mock.MagicMock(spec=acls._PerOperationACL)
        type(poacl).users = mock.PropertyMock(return_value=[service_user_id])
        acl.get.return_value = poacl
        mock_aclm.return_value = acl

        acl_auth_object = barbican_acl.BarbicanACLAuth()
        acl_auth_object.revoke_secret_access(mock.Mock(), mock.Mock())
        acl.submit.assert_called_once()

    @mock.patch('octavia.common.keystone.KeystoneSession')
    def test_get_barbican_client_user_auth(self, mock_ksession):
        acl_auth_object = barbican_acl.BarbicanACLAuth()
        bc = acl_auth_object.get_barbican_client_user_auth(mock.Mock())
        self.assertTrue(hasattr(bc, 'containers') and
                        hasattr(bc.containers, 'register_consumer'))
        self.assertEqual('publicURL', bc.client.interface)
        self.assertEqual('RegionOne', bc.client.region_name)
